from __future__ import annotations

from unittest import mock
from pathlib import Path
import shutil
from copy import deepcopy

from detect_secrets import SecretsCollection
from detect_secrets.core.potential_secret import PotentialSecret
from detect_secrets.settings import transient_settings
from pytest_mock import MockerFixture

from checkov.secrets.git_history_store import GitHistorySecretStore
from checkov.secrets.git_types import Commit, CommitMetadata
from checkov.secrets.runner import Runner
from checkov.runner_filter import RunnerFilter
from checkov.common.output.secrets_record import COMMIT_REMOVED_STR, COMMIT_ADDED_STR

from tests.secrets.git_history.test_utils import mock_git_repo_commits1, mock_git_repo_commits2, \
    mock_git_repo_commits3, mock_git_repo_commits_remove_file, mock_git_repo_commits_rename_file, \
    mock_git_repo_commits_modify_and_rename_file, mock_remove_file_with_two_equal_secret, \
    mock_remove_file_with_two_secret, mock_git_repo_multiline_json, mock_git_repo_multiline_terraform, \
    mock_git_repo_multiline_yml, mock_commit_with_keyword_combinator, mock_set_repo, mock_get_first_commit, \
    mock_get_first_empty_commit, mock_run_forever


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_git_repo_commits1)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_commit)
def test_scan_git_history() -> None:
    valid_dir_path = "test"

    runner = Runner()
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 6
    assert len(report.parsing_errors) == 0
    assert len(report.passed_checks) == 0
    assert len(report.parsing_errors) == 0
    assert len(report.skipped_checks) == 0
    for failed_check in report.failed_checks:
        assert failed_check.added_commit_hash or failed_check.removed_commit_hash
        if failed_check.removed_commit_hash:
            assert failed_check.removed_date
        assert failed_check.added_by
        assert failed_check.added_date


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_git_repo_commits1)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_commit)
def test_scan_history_secrets() -> None:
    valid_dir_path = "test"
    secrets = SecretsCollection()
    plugins_used = [
        {'name': 'AWSKeyDetector'},
    ]
    from checkov.secrets.scan_git_history import GitHistoryScanner

    with transient_settings({
        # Only run scans with only these plugins.
        'plugins_used': plugins_used
    }) as settings:
        settings.disable_filters(*['detect_secrets.filters.common.is_invalid_file'])
        GitHistoryScanner(valid_dir_path, secrets).scan_history()
    assert len(secrets.data) == 3


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_git_repo_commits2)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_commit)
def test_scan_git_history_merge_added_removed() -> None:
    """
    add, move, remove, add, move = secret with the first added_commit_hash and not removed_commit_hash
    """
    valid_dir_path = "test"

    runner = Runner()
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 4
    for failed_check in report.failed_checks:
        assert failed_check.removed_commit_hash == ''
        assert failed_check.removed_date == ''
        assert failed_check.added_commit_hash == '11e59e4e578c6ebcb48aae1e5e078a54c62920eb' or failed_check.added_commit_hash == 'c9b3268e15eb20fd406b9077a4c45875086d6c1b'
        assert failed_check.added_by
        assert failed_check.added_date


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_git_repo_commits2)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_commit)
def test_scan_history_secrets_merge_added_removed() -> None:
    valid_dir_path = "test"
    secrets = SecretsCollection()
    plugins_used = [
        {'name': 'AWSKeyDetector'},
    ]
    from checkov.secrets.scan_git_history import GitHistoryScanner

    with transient_settings({
        # Only run scans with only these plugins.
        'plugins_used': plugins_used
    }) as settings:
        settings.disable_filters(*['detect_secrets.filters.common.is_invalid_file'])
        GitHistoryScanner(valid_dir_path, secrets).scan_history()
    assert len(secrets.data) == 1


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_git_repo_commits3)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_commit)
def test_scan_git_history_merge_added_removed2() -> None:
    """
        add, move, add, remove one = 2 secret one with removed_commit_hash && added_commit_hash
        and one with only added_commit_hash
    """
    valid_dir_path = "/Users/lshindelman/development/test2"

    runner = Runner()
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 5
    assert ((report.failed_checks[0].removed_commit_hash == '697308e61171e33224757e620aaf67b1a877c99d'
             and report.failed_checks[0].removed_date
             and report.failed_checks[1].removed_commit_hash == '')
            or (report.failed_checks[1].removed_commit_hash == '697308e61171e33224757e620aaf67b1a877c99d'
                and report.failed_checks[1].removed_date
                and report.failed_checks[0].removed_commit_hash == ''))
    assert report.failed_checks[0].added_by and report.failed_checks[0].added_date
    assert report.failed_checks[1].added_by and report.failed_checks[1].added_date
    assert ((report.failed_checks[0].added_commit_hash == '900b1e8f6f336a92e8f5fca3babca764e32c3b3d'
             and report.failed_checks[1].added_commit_hash == '3c8cb7eedb3986308c96713fc65b006adcf3bc44')
            or (report.failed_checks[1].added_commit_hash == '900b1e8f6f336a92e8f5fca3babca764e32c3b3d'
                and report.failed_checks[0].added_commit_hash == '3c8cb7eedb3986308c96713fc65b006adcf3bc44'))
    # print testing
    assert_for_commit_str(report.failed_checks[0].to_string() + report.failed_checks[1].to_string(),
                          commit_type=COMMIT_REMOVED_STR,
                          commit_hash='697308e61171e33224757e620aaf67b1a877c99d')
    assert_for_commit_str(report.failed_checks[0].to_string() + report.failed_checks[1].to_string(),
                          commit_type=COMMIT_ADDED_STR,
                          commit_hash='3c8cb7eedb3986308c96713fc65b006adcf3bc44')
    assert_for_commit_str(report.failed_checks[0].to_string() + report.failed_checks[1].to_string(),
                          commit_type=COMMIT_ADDED_STR,
                          commit_hash='900b1e8f6f336a92e8f5fca3babca764e32c3b3d')


# this test is too flaky !
# @pytest.mark.filterwarnings("error")  # otherwise pytest sometimes suppresses the raised Timeout Exception
# @mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_git_repo_commits_too_much)
# @mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
# @mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_first_commit', mock_get_first_commit)
# def test_scan_history_secrets_timeout() -> None:
#     """
#     add way too many cases to check in 1 second
#     """
#     valid_dir_path = "test"
#     secrets = SecretsCollection()
#     plugins_used = [
#         {'name': 'AWSKeyDetector'},
#     ]
#     from checkov.secrets.scan_git_history import GitHistoryScanner
#
#     with transient_settings({
#         # Only run scans with only these plugins.
#         'plugins_used': plugins_used
#     }) as settings:
#         settings.disable_filters(*['detect_secrets.filters.common.is_invalid_file'])
#         finished = GitHistoryScanner(valid_dir_path, secrets, None, 1).scan_history()
#
#     assert finished is False


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_run_forever)
def test_scan_history_secrets_timeout() -> None:
    """
    add way too many cases to check in 1 second
    """
    valid_dir_path = "test"
    secrets = SecretsCollection()
    plugins_used = [
        {'name': 'AWSKeyDetector'},
    ]
    from checkov.secrets.scan_git_history import GitHistoryScanner

    with transient_settings({
        # Only run scans with only these plugins.
        'plugins_used': plugins_used
    }) as settings:
        settings.disable_filters(*['detect_secrets.filters.common.is_invalid_file'])
        finished = GitHistoryScanner(valid_dir_path, secrets, None, 1).scan_history()

    assert finished is False


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_git_repo_commits_remove_file)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_commit)
def test_scan_git_history_remove_file() -> None:
    valid_dir_path = "remove_file"

    runner = Runner()
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 4
    assert report.failed_checks[0].removed_commit_hash == '4bd08cd0b2874025ce32d0b1e9cd84ca20d59ce1'
    assert report.failed_checks[0].removed_date
    assert report.failed_checks[0].added_commit_hash == '63342dbee285973a37770bbb1ff4258a3184901e'
    assert report.failed_checks[0].added_by and report.failed_checks[0].added_date
    assert report.failed_checks[0].removed_date


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_git_repo_commits_rename_file)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_commit)
def test_scan_git_history_rename_file() -> None:
    valid_dir_path = "/test/git/history/rename/file"

    runner = Runner()
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 5
    assert (report.failed_checks[0].removed_commit_hash == '' and report.failed_checks[0].removed_date == '' and
            report.failed_checks[0].added_commit_hash == '2e1a500e688990e065fc6f1202bc64ed0ba53027')
    assert (report.failed_checks[1].removed_commit_hash == '2e1a500e688990e065fc6f1202bc64ed0ba53027' and
            report.failed_checks[1].removed_date == '2022-12-14T16:32:13+00:00' and
            report.failed_checks[1].added_commit_hash == 'adef7360b86c62666f0a70521214220763b9c593')


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff',
            mock_git_repo_commits_modify_and_rename_file)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_commit)
def test_scan_git_history_modify_and_rename_file() -> None:
    valid_dir_path = "test_scan_git_history_modify_and_rename_file"

    runner = Runner()
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 4
    assert report.failed_checks[0].added_commit_hash == '62da8e5e04ec5c3a474467e9012bf3427cff0407'
    assert report.failed_checks[0].added_by and report.failed_checks[0].added_date
    assert report.failed_checks[0].removed_commit_hash == '61ee79aea3d151a40c8e054295f330d233eaf7d5'
    assert report.failed_checks[0].removed_date


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff',
            mock_remove_file_with_two_equal_secret)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_empty_commit)
def test_scan_git_history_rename_file_with_two_equal_secrets() -> None:
    valid_dir_path = "test_scan_git_history_rename_file_with_two_equal_secrets"
    runner = Runner()
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 2

    assert report.failed_checks[0].removed_commit_hash == report.failed_checks[1].removed_commit_hash
    assert report.failed_checks[1].removed_commit_hash
    assert report.failed_checks[0].removed_date and report.failed_checks[0].removed_date == report.failed_checks[
        1].removed_date


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_remove_file_with_two_secret)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_empty_commit)
def test_scan_git_history_rename_file_with_two_secrets() -> None:
    valid_dir_path = "test_scan_git_history_rename_file_with_two_secrets"
    runner = Runner()
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 2
    report.failed_checks[0].removed_commit_hash == report.failed_checks[1].removed_commit_hash
    assert report.failed_checks[1].removed_commit_hash
    assert report.failed_checks[0].removed_date and report.failed_checks[0].removed_date == report.failed_checks[
        1].removed_date


def assert_for_commit_str(report_str: [str], commit_type: str, commit_hash: str, found: bool = True) -> None:
    to_find = f'; {commit_type}: {commit_hash}'
    assert (to_find in report_str) == found


# added all file scenarios from multiline tests
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff',
            mock_git_repo_multiline_json)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_empty_commit)
def test_scan_git_history_multiline_keyword_json() -> None:
    valid_dir_path = "multiline_keyword_json"

    runner = Runner()
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 5
    assert report.parsing_errors == []
    assert report.passed_checks == []
    assert report.skipped_checks == []


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_git_repo_multiline_terraform)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_empty_commit)
def test_scan_git_history_multiline_keyword_terraform() -> None:
    valid_dir_path = "mock_git_repo_multiline_terraform"

    runner = Runner()
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    #  then
    failing_resources = {
        "6bee3eb2f69e06095395ae1d54c810c3a2a99841:9ed4f1457a9c27dd868c1f21276c6d7098d2bacf",
        "6bee3eb2f69e06095395ae1d54c810c3a2a99841:5db2fafebcfed9b4c9ffc570c46ef2ca94a3881a",
        "6bee3eb2f69e06095395ae1d54c810c3a2a99841:ac236b0474a9a702f99dbe244a14548783ace5c5",
        "6bee3eb2f69e06095395ae1d54c810c3a2a99841:06af723e58378574456be0b4c41a89194aaed0c3",
        "6bee3eb2f69e06095395ae1d54c810c3a2a99841:dcbf46de362e1b6942054b89ee293984e9a8a40a",
    }

    failed_check_resources = {c.resource for c in report.failed_checks}

    assert len(report.failed_checks) == len(failing_resources)
    assert report.parsing_errors == []
    assert report.passed_checks == []
    assert report.skipped_checks == []
    assert failing_resources == failed_check_resources


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff', mock_git_repo_multiline_yml)
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_commit)
def test_scan_git_history_multiline_keyword_yml() -> None:
    valid_dir_path = "mock_git_repo_multiline_yml"
    runner = Runner()

    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 8
    assert report.parsing_errors == []
    assert report.passed_checks == []
    assert report.skipped_checks == []


def test_scan_git_history_full_vs_partial(mocker: MockerFixture) -> None:
    # this takes the 3 mock commits and run _test_it on them
    commits_func = [mock_git_repo_commits1('', ''), mock_git_repo_commits2('', ''), mock_git_repo_commits3('', '')]
    all([_test_it(mocker, commits) for commits in commits_func])


def _test_it(mocker: MockerFixture, all_commits) -> bool:
    """
    this test tries to run a full scan over 5 commits,
    then run two separate runs over the first 2 and the last 3 (the second will give the secret store to the third)
    then compares the results from run 1 to the last run
    """
    valid_dir_path = "test"
    mocker.patch(
        "checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff",
        return_value=all_commits,
    )
    runner = Runner()
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    sec_store = runner.get_history_secret_store()
    # assert len(report.failed_checks) == 3
    for failed_check in report.failed_checks:
        assert failed_check.added_commit_hash or failed_check.removed_commit_hash
        assert failed_check.added_by and failed_check.added_date

    mocker.patch(
        "checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff", return_value=all_commits[0:1])
    runner2 = Runner()
    report2 = runner2.run(root_folder=valid_dir_path, external_checks_dir=None,
                          runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    # assert len(report2.failed_checks) == 1
    sec_store2 = runner2.get_history_secret_store()
    sec_store2_dc = deepcopy(sec_store2)

    mocker.patch(
        "checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff", return_value=all_commits[2:5])
    runner3 = Runner()
    runner3.set_history_secret_store(sec_store2_dc)
    report3 = runner3.run(root_folder=valid_dir_path, external_checks_dir=None,
                          runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    sec_store3 = runner3.get_history_secret_store()
    assert len(report3.failed_checks) == len(report.failed_checks)
    for failed_check in report3.failed_checks:
        assert failed_check.added_commit_hash or failed_check.removed_commit_hash
        assert failed_check.added_by and failed_check.added_date
    # check the secret store to have the same results
    assert len(sec_store) == len(sec_store3)
    for k1, k3 in zip(sec_store, sec_store3):
        assert k1 == k3
        assert sec_store[k1][0].get('added_commit_hash') == sec_store3[k3][0].get('added_commit_hash')
    return True


def test_scan_git_history_real_repo() -> None:
    """
    runs over a real repo inside the resource dir and takes the results
    """

    dir_path = Path(__file__).parent / 'git_history/testing_repo'
    git_conf_dir = dir_path / 'git_to_change'
    tmp_git_conf_dir = dir_path / '.git'
    shutil.rmtree(tmp_git_conf_dir, ignore_errors=True)  # make sure no left overs from prev run
    shutil.copytree(git_conf_dir, tmp_git_conf_dir)

    runner = Runner()
    report = runner.run(root_folder=str(dir_path), external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 2
    assert report.failed_checks[0].added_commit_hash and not report.failed_checks[0].removed_commit_hash
    assert report.failed_checks[0].added_date and not report.failed_checks[0].removed_date
    assert report.failed_checks[1].added_commit_hash and report.failed_checks[1].removed_commit_hash and \
           report.failed_checks[1].removed_date
    shutil.rmtree(tmp_git_conf_dir)  # just for cleaning


@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_commit)
def test_git_history_plugin(mocker: MockerFixture) -> None:
    valid_dir_path = "test"
    commits = mock_commit_with_keyword_combinator()
    mocker.patch(
        "checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff", return_value=commits)
    runner = Runner()
    report = runner.run(root_folder=str(valid_dir_path), external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], enable_git_history_secret_scan=True))
    assert len(report.failed_checks) == 4
    check = report.failed_checks[0]
    assert check.added_commit_hash
    assert check.check_name == 'Base64 High Entropy String'


@mock.patch("checkov.secrets.scan_git_history.GitHistoryScanner._get_commits_diff", lambda self, last_commit_sha: [])
@mock.patch('checkov.secrets.scan_git_history.GitHistoryScanner.set_repo', mock_set_repo)
@mock.patch('checkov.secrets.scan_git_history.get_first_commit', mock_get_first_commit)
def test_scan_history_secrets_with_history_store_and_no_new_commit() -> None:
    # given
    root_folder = "test"
    secrets = SecretsCollection()
    plugins_used = [
        {"name": "AWSKeyDetector"},
    ]

    file_name = "Dockerfile"
    file_results = [
        PotentialSecret(
            type="AWS Access Key",
            filename=file_name,
            secret="wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY",
            line_number=7,
            is_added=True,
            is_removed=False,
        )
    ]
    commit = Commit(
        metadata=CommitMetadata(
            commit_hash="8a21fa691e17907afee57e93b7820c5943b12746",
            committer="Momo",
            committed_datetime="2022-12-24T01:02:03+00:00",
        ),
        files={
            "Dockerfile": 'diff --git a/Dockerfile b/Dockerfile\nindex 0000..0000 0000\n--- a/Dockerfile\n+++ b/Dockerfile\n@@ -4,6 +4,8 @@ FROM public.ecr.aws/lambda/python:3.9\n \n ENV PIP_ENV_VERSION="2022.1.8"\n \n+ENV AWS_ACCESS_KEY_ID="wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY"\n+\n COPY Pipfile Pipfile.lock ./\n \n RUN pip install pipenv==${PIP_ENV_VERSION} \\\n'
        },
    )

    history_store = GitHistorySecretStore()
    history_store.set_secret_map(file_results=file_results, file_name=file_name, commit=commit)

    # when
    from checkov.secrets.scan_git_history import GitHistoryScanner

    with transient_settings(
            {
                # Only run scans with only these plugins.
                "plugins_used": plugins_used
            }
    ) as settings:
        settings.disable_filters(*["detect_secrets.filters.common.is_invalid_file"])
        GitHistoryScanner(root_folder=root_folder, secrets=secrets, history_store=history_store).scan_history()

    # then
    assert len(secrets.data) == 1



from constructs import Construct
from aws_cdk import App, Stack 
from aws_cdk import (
    aws_s3 as s4
)


class MyS3Stack(Stack):

    def __init__(self, scope: Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        bucket = s4.Bucket(self, "MyBlockedBucket",
            block_public_access=s4.BlockPublicAccess(block_public_policy=True)
        )

        bucket2 = s4.Bucket(self, "MyBlockedBucket2",
                        block_public_access=s4.BlockPublicAccess.BLOCK_ALL
                    )

app = App()
MyS3Stack(app, "MyS3Stack")
app.synth()


import os
import unittest

from checkov.arm.checks.resource.MySQLServerSSLEnforcementEnabled import check
from checkov.arm.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestMySQLServerSSLEnforcementEnabled(unittest.TestCase):

    def test_summary(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = current_dir + "/example_MySQLServerSSLEnforcementEnabled"
        report = runner.run(root_folder=test_files_dir,runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        self.assertEqual(summary['passed'], 1)
        self.assertEqual(summary['failed'], 2)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)


if __name__ == '__main__':
    unittest.main()



from checkov.secrets.runner import should_filter_vault_secret

HIGH_ENTROPY_CHECK_ID = 'CKV_SECRET_80'

def test_vault_secrets_false_positives():
    fp_secrets = [
        'DB_RBMQ_PASSWORD: vault: secret/data/product-web/mcrp-qwr-v2/mabbot#PASSWORD',
        'WEB_PASSWORD: vault: secret/data/product/fwrp-qe-v3/parme3#PASSWORD',
        'PASS: vault: secret/sr/dt/pro/fwrtq1#2/weg#PASSWORD'
    ]
    for fp_secret in fp_secrets:
        assert should_filter_vault_secret(fp_secret, HIGH_ENTROPY_CHECK_ID)

def test_secrets_without_vault():
    real_secrets = [
        'ldap_pwd = k%udk423u4%P8=H_',
        'password = J6T4ww+##14m',
        'PS = 1r4#Gf2FDF$343r3m2me3r%'
    ]
    for real_secret in real_secrets:
        assert not should_filter_vault_secret(real_secret, HIGH_ENTROPY_CHECK_ID)


import unittest

import hcl2

from checkov.terraform.checks.resource.gcp.GoogleComputeIPForward import check
from checkov.common.models.enums import CheckResult


class TestGoogleComputeIPForward(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance" "default" {
              name         = "test"
              machine_type = "n1-standard-1"
              zone         = "us-central1-a"
              can_ip_forward = true
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_1(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_template" "default" {
              name         = "test"
              machine_type = "n1-standard-1"
              zone         = "us-central1-a"
              can_ip_forward = true
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_template']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_2(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_from_template" "default" {
              name         = "test"
              source_instance_template = google_compute_instance_template.default.id
              can_ip_forward = true
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_from_template']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance" "default" {
              name         = "test"
              machine_type = "n1-standard-1"
              zone         = "us-central1-a"
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_2(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance" "default" {
              name         = "gke-test"
              machine_type = "n1-standard-1"
              zone         = "us-central1-a"
              can_ip_forward = true
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_3(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_template" "default" {
              name         = "test"
              machine_type = "n1-standard-1"
              zone         = "us-central1-a"
              boot_disk {}
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_template']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_4(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_from_template" "default" {
              name         = "test"
              source_instance_template = google_compute_instance_template.default.id
              can_ip_forward = false

            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_from_template']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_unknown(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_from_template" "default" {
              name         = "test"
              source_instance_template = google_compute_instance_template.default.id
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_from_template']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.UNKNOWN, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.KeyvaultRecoveryEnabled import check
from checkov.common.models.enums import CheckResult


class TestKeyVaultRecoveryEnabled(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_key_vault" "example" {
              name                        = "testvault"
              location                    = azurerm_resource_group.example.location
              resource_group_name         = azurerm_resource_group.example.name
              enabled_for_disk_encryption = true
              tenant_id                   = data.azurerm_client_config.current.tenant_id
              sku_name = "standard"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure2(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_key_vault" "example" {
              name                        = "testvault"
              location                    = azurerm_resource_group.example.location
              resource_group_name         = azurerm_resource_group.example.name
              enabled_for_disk_encryption = true
              tenant_id                   = data.azurerm_client_config.current.tenant_id
              soft_delete_enabled         = false
              sku_name = "standard"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_key_vault" "example" {
              name                        = "testvault"
              location                    = azurerm_resource_group.example.location
              resource_group_name         = azurerm_resource_group.example.name
              enabled_for_disk_encryption = true
              tenant_id                   = data.azurerm_client_config.current.tenant_id
              soft_delete_enabled         = true
              purge_protection_enabled    = true
              sku_name = "standard"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success2(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_key_vault" "example" {
              name                        = "testvault"
              location                    = azurerm_resource_group.example.location
              resource_group_name         = azurerm_resource_group.example.name
              enabled_for_disk_encryption = true
              tenant_id                   = data.azurerm_client_config.current.tenant_id
              purge_protection_enabled    = true
              sku_name = "standard"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import pytest

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.data.base_check import BaseDataCheck


class TestStaticCheck(BaseDataCheck):
    # for pytest not to collect this class as tests
    __test__ = False

    def __init__(self):
        name = "Test something"
        id = "CKV_TEST_2"
        supported_data = ["ckv_test"]
        categories = [CheckCategories.CONVENTION]
        super().__init__(name=name, id=id, categories=categories, supported_data=supported_data)

    def scan_data_conf(self, conf):
        if "check_result" in conf.keys():
            check_result = conf["check_result"][0]
            if check_result:
                return CheckResult.PASSED

            return CheckResult.FAILED

        return CheckResult.UNKNOWN


@pytest.mark.parametrize(
    "conf,expected",
    [
        ({"check_result": [True]}, CheckResult.PASSED),
        ({"check_result": [False]}, CheckResult.FAILED),
        ({"foo": ["bar"]}, CheckResult.UNKNOWN),
        ({"count": [0], "check_result": [True]}, CheckResult.UNKNOWN),
        ({"count": [1], "check_result": [True]}, CheckResult.PASSED),
    ],
    ids=["pass", "fail", "unknown", "count_zero", "count_one"],
)
def test_scan_entity_conf(conf, expected):
    result = TestStaticCheck().scan_entity_conf(conf, "ckv_test")

    assert result == expected



import os
import unittest

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.terraform.checks.resource.registry import resource_registry as registry
from checkov.terraform.runner import Runner
from checkov.runner_filter import RunnerFilter


class TerraformCheck(BaseResourceCheck):

    def __init__(self):
        name = "Terraform test"
        id = "CKV_T_1"
        supported_resources = ['aws_iam_*', 'null_resource', '*s3*']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        return CheckResult.PASSED


class TestWildcardEntities(unittest.TestCase):

    def test_summary(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        check = TerraformCheck()

        test_files_dir = current_dir + "/example_WildcardEntities"
        report = runner.run(root_folder=test_files_dir, runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        registry.wildcard_checks['aws_iam_*'].remove(check)
        registry.checks['null_resource'].remove(check)

        registry.wildcard_checks['*s3*'].remove(check)
        
        del registry.checks['null_resource']
        del registry.wildcard_checks['*s3*']
        del registry.wildcard_checks['aws_iam_*']
        # Only for resource and nof for data "aws_iam_policy_document"
        self.assertEqual(summary['passed'], 3)
        self.assertEqual(summary['failed'], 0)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)


if __name__ == '__main__':
    unittest.main()



import os
from unittest import TestCase, mock
from datetime import datetime

import pytest

from checkov.terraform.graph_builder.variable_rendering.evaluate_terraform import evaluate_terraform, \
    replace_string_value, \
    remove_interpolation, _find_new_value_for_interpolation
from checkov.terraform.graph_builder.variable_rendering.safe_eval_functions import evaluate, get_asteval


class TestTerraformEvaluation(TestCase):
    def test_zipmap(self):
        input_str = '"zipmap(["a", "b"], [1, 2])"'
        expected = {'a': 1, 'b': 2}
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_directive(self):
        input_str = '"Hello, %{ if "d" != "" }named%{ else }unnamed%{ endif }!"'
        expected = 'Hello, named!'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_condition(self):
        input_str = '"2 > 0 ? bigger : smaller"'
        expected = 'bigger'
        self.assertEqual(expected, evaluate_terraform(input_str).strip())

        input_str = '"2 > 5 ? bigger : smaller"'
        expected = 'smaller'
        self.assertEqual(expected, evaluate_terraform(input_str).strip())

    def test_conditional_expression(self):
        input_str = '"[\'${blocked == "allowed" ? True : False}\']"'
        expected = False
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = '${blocked == "allowed" ? True : False}'
        expected = False
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'blocked == "allowed" ? True : False'
        expected = False
        self.assertEqual(expected, evaluate_terraform(input_str))
        
        input_str = 'True == "true" ? True : False'
        expected = True
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'False != "false" ? True : False'
        expected = False
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_nested_conditional_expression(self):
        input_str = "{for resource in concat(true ? [{'name'='test'}] : [], false ? [] : [{'name'='test2'}]) : resource.name => resource}"
        value = evaluate_terraform(input_str)
        self.assertEqual(value, {'test': {'name': 'test'}, 'test2': {'name': 'test2'}})


    def test_format(self):
        input_str = '"format("Hello, %s!", "Ander")"'
        expected = 'Hello, Ander!'
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = '"format("There are %d lights", 4)"'
        expected = 'There are 4 lights'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_formatlist(self):
        input_str = '"formatlist("Hello, %s!", ["Valentina", "Ander", "Olivia", "Sam"])"'
        expected = ['Hello, Valentina!', 'Hello, Ander!', 'Hello, Olivia!', 'Hello, Sam!']
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_join(self):
        input_str = 'join(", ", ["foo", "bar", "baz"])'
        expected = 'foo, bar, baz'
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'join(", ", ["foo"])'
        expected = 'foo'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_regex(self):
        input_str = 'regex("[a-z]+", "53453453.345345aaabbbccc23454")'
        expected = 'aaabbbccc'
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'regex("[a-z]+", "53453453.34534523454")'
        expected = ''
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'regex("^(?:(?P<scheme>[^:/?#]+):)?(?://(?P<authority>[^/?#]*))?", "https://terraform.io/docs/")'
        expected = {"authority":"terraform.io", "scheme": "https"}
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'regex(r"(\\d\\d\\d\\d)-(\\d\\d)-(\\d\\d)", "2019-02-01")'
        expected = ["2019","02","01"]
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'regex("[a-z]+", "53453453.34534523454")'
        expected = ''
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_regexall(self):
        input_str = 'regexall("[a-z]+", "1234abcd5678efgh9")'
        expected = ["abcd","efgh"]
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'length(regexall("[a-z]+", "1234abcd5678efgh9"))'
        expected = 2
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'length(regexall("[a-z]+", "123456789")) > 0'
        expected = False
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_replace(self):
        input_str = 'replace("1 + 2 + 3", "+", "-")'
        expected = -4
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_substr(self):
        input_str = 'substr("hello world", 1, 4)'
        expected = 'ello'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_trim(self):
        input_str = 'trim("?!hello?!", "!?")'
        expected = 'hello'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_trimprefix(self):
        input_str = 'trimprefix("helloworld", "hello")'
        expected = 'world'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_upper(self):
        input_str = 'upper("hello")'
        expected = 'HELLO'
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'upper("алло!")'
        expected = 'АЛЛО!'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_chunklist(self):
        input_str = 'chunklist(["a", "b", "c", "d", "e"], 2)'
        expected = [["a", "b"], ["c", "d"], ["e"]]
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_coalese(self):
        input_str = 'coalesce("a", "b")'
        expected = 'a'
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'coalesce("", "b")'
        expected = 'b'
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'coalesce(1, 2)'
        expected = 1
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_coalescelist(self):
        input_str = 'coalescelist(["a", "b"], ["c", "d"])'
        expected = ['a', 'b']
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'coalescelist([], ["c", "d"])'
        expected = ["c", "d"]
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_compact(self):
        input_str = 'compact(["a", "", "b", "c"])'
        expected = ['a', 'b', 'c']
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_concat(self):
        input_str = 'concat(["a", ""], ["b", "c"])'
        expected = ['a', '', 'b', 'c']
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'concat([\'postgresql-tcp\'],[],[\'\'])'
        expected = ['postgresql-tcp', '']
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_concat_dictionaries(self):
        input_str = "concat([{'key':'a','value':'a'},{'key':'b','value':'b'}, \"{'key':'d','value':'d'}\"],,[{'key':'c','value':'c'}],)"
        expected = [{'key':'a','value':'a'},{'key':'b','value':'b'},"{'key':'d','value':'d'}",{'key':'c','value':'c'}]
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'concat([\'postgresql-tcp\'],[],[\'\'])'
        expected = ['postgresql-tcp', '']
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_distinct(self):
        input_str = 'distinct(["a", "b", "a", "c", "d", "b"])'
        expected = ['a', 'b', 'c', 'd']
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_flatten(self):
        input_str = 'flatten([["a", "b"], [], ["c"]])'
        expected = ['a', 'b', 'c']
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = 'flatten([[["a", "b"], []], ["c"]])'
        expected = ['a', 'b', 'c']
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_index(self):
        input_str = 'index(["a", "b", "c"], "b")'
        expected = 1
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_keys(self):
        input_str = 'keys({"a"="ay", "b"="bee"})'
        expected = ["a", "b"]
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_list(self):
        input_str = 'list("a", "b", "c")'
        expected = ['a', 'b', 'c']
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_lookup(self):
        input_str = 'lookup({"a"="ay", "b"="bee"}, "a", "what?")'
        expected = 'ay'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_matchkeys(self):
        input_str = 'matchkeys(["i-123", "i-abc", "i-def"], ["us-west", "us-east", "us-east"], ["us-east"])'
        expected = ["i-abc", "i-def"]
        actual = evaluate_terraform(input_str)
        for elem in actual:
            if elem not in expected:
                self.fail(f'expected to find {elem} in {expected}. Got {actual}')

    def test_merge(self):
        input_str = 'merge({"a"="b", "c"="d"}, {"e"="f", "c"="z"})'
        expected = {"a":"b", "c":"z","e":"f"}
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_merge2(self):
        input_str = 'merge({"a"="b", "c"="d"}, {"e"="f", "c"="z"}, {"r"="o", "t"="m"})'
        expected = {"a":"b", "c":"z","e":"f","r":"o","t":"m"}
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_merge_multiline(self):
        input_str = "merge(\n{'Tag1':'one','Tag2':'two'},\n{'Tag4' = 'four'},\n{'Tag2'='multiline_tag2'})"
        expected = {'Tag1': 'one', 'Tag2': 'multiline_tag2', 'Tag4': 'four'}
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_merge_interpolation(self):
        input_str = '${merge({\'environment\':\'${var.environment}\',\'name\':\'${local.cluster_name}\',\'role\':\'${var.role}\',\'team\':\'${var.team}\'})}'
        expected = {'environment': 'var.environment', 'name': 'local.cluster_name', 'role': 'var.role', 'team': 'var.team'}
        actual = evaluate_terraform(input_str, keep_interpolations=False)
        self.assertEqual(expected, actual)


    def test_reverse(self):
        input_str = 'reverse([1, 2, 3])'
        expected = [3, 2, 1]
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_sort(self):
        input_str = 'sort(compact(distinct(concat([\'postgresql-tcp\'],[],[\'\']))))'
        expected = ['postgresql-tcp']
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_condition2(self):
        input_str = 'us-west-2 == "something to produce false" ? true : false'
        expected = 'false'
        self.assertEqual(expected, evaluate_terraform(input_str).strip())

    def test_complex_merge(self):
        cases = [
            ("merge(local.one, local.two)", "merge(local.one, local.two)"),
            ('merge({"Tag4" = "four"}, {"Tag5" = "five"})', {"Tag4": "four", "Tag5": "five"}),
            ('merge({"a"="b"}, {"b"=[1,2], "c"="z"}, {"d"=3})', {"a": "b", "b": [1, 2], "c": "z", "d": 3}),
            ("merge({'a': '}, evil'})", {"a": "}, evil"}),
            (
                "merge(local.common_tags,,{'Tag4': 'four'},,{'Tag2': 'Dev'},)",
                "merge(local.common_tags,{'Tag4': 'four'},{'Tag2': 'Dev'},)",
            ),
        ]
        for case in cases:
            input_str = case[0]
            expected = input_str if case[1] is None else case[1]
            actual = evaluate_terraform(input_str)
            assert actual == expected, f'Case "{input_str}" failed. Expected: {expected}  Actual: {actual}'

    def test_map_access(self):
        input_str = '{\'module-input-bucket\':\'mapped-bucket-name\'}[module-input-bucket]-works-yay'
        expected = 'mapped-bucket-name-works-yay'
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = '{"module-input-bucket":"mapped-bucket-name"}[module-input-bucket]-works-yay'
        expected = 'mapped-bucket-name-works-yay'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_replace_with_map(self):
        original_str = '{\'module-input-bucket\':\'mapped-bucket-name\'}[module.bucket.name]-works-yay'
        replaced = replace_string_value(original_str, "module.bucket.name", "module-input-bucket", keep_origin=False)
        expected = '{\'module-input-bucket\':\'mapped-bucket-name\'}[module-input-bucket]-works-yay'
        self.assertEqual(expected, replaced)

    def test_replace_interpolation(self):
        original_str = '${mapped-bucket-name}[module.bucket.name]-works-yay'
        replaced = replace_string_value(original_str, "module.bucket.name", "module-input-bucket", keep_origin=False)
        expected = '${mapped-bucket-name}[module-input-bucket]-works-yay'
        self.assertEqual(expected, replaced)

    def test_remove_interpolation1(self):
        original_str = '${merge(local.common_tags,local.common_data_tags,{\'Name\':\'Bob-${local.static1}-${local.static2}\'})}'
        replaced = remove_interpolation(original_str)
        expected = 'merge(local.common_tags,local.common_data_tags,{\'Name\':\'Bob-local.static1-local.static2\'})'
        self.assertEqual(expected, replaced)

    def test_jsonencode(self):
        cases = [
            ("jsonencode(['a', 42, true, null])", ["a", 42, True, None]),
            ("jsonencode({'a': 'b'})", {"a": "b"}),
            ("jsonencode({'a' = 'b'})", {"a": "b"}),
            ("jsonencode({'a' = 42})", {"a": 42}),
            ("jsonencode({'a' = true})", {"a": True}),
            ("jsonencode({'a' = false})", {"a": False}),
            ("jsonencode({'a' = null})", {"a": None}),
            ("jsonencode({'a' = ['b', 'c']})", {"a": ["b", "c"]}),
            ("jsonencode({'a' = jsonencode(['b', 'c'])})", {"a": ["b", "c"]}),
        ]

        for input_str, expected in cases:
            with self.subTest(input_str):
                assert evaluate_terraform(input_str) == expected

    def test_block_file_write(self):
        temp_file_path = "/tmp/file_shouldnt_create"
        input_str = "[x for x in {}.__class__.__bases__[0].__subclasses__() if x.__name__ == 'catch_warnings'][0]()._module.__builtins__['__import__']('os').system('date >> /tmp/file_shouldnt_create')"
        evaluated = evaluate_terraform(input_str)
        self.assertEqual(input_str, evaluated)
        self.assertFalse(os.path.exists(temp_file_path))

    def test_block_file_write2(self):
        temp_file_path = "/tmp/file_shouldnt_create_vuln"
        input_str = "(lambda: [x for x in {}.__class__.__bases__[0].__subclasses__() if x.__name__ == 'catch_warnings'][0]()._module.__builtins__['__import__']('os').system('date >> /tmp/file_shouldnt_create_vuln'))()"
        evaluated = evaluate_terraform(input_str)
        self.assertEqual(input_str, evaluated)
        self.assertFalse(os.path.exists(temp_file_path))

    def test_block_file_write_lower(self):
        temp_file_path = "/tmp/file_shouldnt_create"
        input_str = "[x for x in parsint.__bases__[0].__subclasses__()][134]()._module.__builtins__['__IMPORT__'.lower()]('os').system('date >> /tmp/file_shouldnt_create')"
        evaluated = evaluate_terraform(input_str)
        self.assertEqual(input_str, evaluated)
        self.assertFalse(os.path.exists(temp_file_path))

    def test_block_math_expr(self):
        input_str = "__import__('math').sqrt(25)"
        evaluated = evaluate_terraform(input_str)
        self.assertEqual(input_str, evaluated)

    def test_block_segmentation_fault(self):
        # in this test, the following code is causing segmentation fault if evaluated
        input_str = """
(lambda fc=(
    lambda n: [
        c for c in
            ().__class__.__bases__[0].__subclasses__()
            if c.__name__ == n
        ][0]
    ):
    fc("function")(
        fc("code")(
            0,0,0,0,0,b'test',(),(),(),"","",0,b'test'
        ),{}
    )()
)()
"""
        evaluated = evaluate_terraform(input_str)
        self.assertEqual(input_str.replace("\n", ""), evaluated)

    def test_evaluate_(self):
        input_str = '"10\\.0\\.\\0.\\0/8"'
        expected = '10\\.0\\.\\0.\\0/8'
        evaluated = evaluate_terraform(input_str)
        self.assertEqual(expected, evaluated)

    # Date Function
    @mock.patch('checkov.terraform.graph_builder.variable_rendering.safe_eval_functions.datetime')
    def test_timestamp(self,mock_dt):
        testdt = datetime(2018, 5, 13, 7, 44, 12, 0)
        mock_dt.utcnow = mock.Mock(return_value=testdt)
        input_str = 'timestamp()'
        expected = "2018-05-13T07:44:12Z"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_timeadd_hours(self):
        input_str = 'timeadd("2018-05-13T07:44:12Z","24h")'
        expected = "2018-05-14T07:44:12Z"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_timeadd_negative_hours(self):
        input_str = 'timeadd("2018-05-13T07:44:12Z","-24h")'
        expected = "2018-05-12T07:44:12Z"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_timeadd_partialhours(self):
        input_str = 'timeadd("2018-05-13T07:44:12Z","1.5h")'
        expected = "2018-05-13T09:14:12Z"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_timeadd_minutes(self):
        input_str = 'timeadd("2018-05-13T07:44:12Z","16m")'
        expected = "2018-05-13T08:00:12Z"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_timeadd_hours_and_minutes(self):
        input_str = 'timeadd("2018-05-13T07:44:12Z","1h16m")'
        expected = "2018-05-13T09:00:12Z"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_timeadd_hours_and_minutes_and_seconds(self):
        input_str = 'timeadd("2018-05-13T07:44:12Z","1h16m49s")'
        expected = "2018-05-13T09:01:01Z"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_timeadd_hours_and_minutes_and_seconds_milliseconds(self):
        input_str = 'timeadd("2018-05-13T07:44:12Z","1h16m49s1001ms")'
        expected = "2018-05-13T09:01:02Z"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_timeadd_hours_and_minutes_and_seconds_milliseconds_microseconds(self):
        input_str = 'timeadd("2018-05-13T07:44:12Z","1h16m49s1001ms1000001us")'
        expected = "2018-05-13T09:01:03Z"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_formatdatesimple(self):
        input_str = 'formatdate("HH:mm", "2018-01-02T23:12:01Z")'
        expected = '11:12'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_formatdate_simple_and_am(self):
        input_str = 'formatdate("HH:mmaa", "2018-01-02T23:12:01Z")'
        expected = '11:12pm'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_formatdate_more_complex(self):
        input_str = 'formatdate("DD MMM YYYY hh:mm", "2018-01-02T23:12:01Z")'
        expected = '02 Jan 2018 23:12'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_formatdate_with_day(self):
        input_str = 'formatdate("EEE, DD MMM YYYY hh:mm:ss ZZZZZ", "2018-01-02T23:12:01-08:00")'
        expected = 'Tue, 02 Jan 2018 23:12:01 -08:00'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_formatdate_utc_and_zzz(self):
        input_str = 'formatdate("DD MMM YYYY hh:mm ZZZ", "2018-01-02T23:12:01Z")'
        expected = '02 Jan 2018 23:12 UTC'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_formatdate_utc_and_z(self):
        input_str = 'formatdate("DD MMM YYYY hh:mm Z", "2018-01-02T23:12:01Z")'
        expected = '02 Jan 2018 23:12 Z'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_formatdate_with_day_utc(self):
        input_str = 'formatdate("EEE, DD MMM YYYY hh:mm:ss ZZZ", "2018-01-02T23:12:01-00:00")'
        expected = 'Tue, 02 Jan 2018 23:12:01 UTC'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_formatdate_everything(self):
        input_str = 'formatdate("YYYY YY MMMM MMM MM M DD EEEE EEE hh h HH H AA aa mm m ss s ZZZZZ ZZZZ ZZZ Z", "2018-01-02T23:12:01-00:00")'
        expected = '2018 18 January Jan 01 1 02 Tuesday Tue 23 23 11 11 PM pm 12 12 01 1 +00:00 +0000 UTC Z'
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_formatdate_simple_and_quotes(self):
        input_str = 'formatdate("HH \'o\'\'clock\'", "2018-01-02T23:12:01Z")'
        expected = "11 o'clock"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_formatdate_simple_and_more_quotes(self):
        input_str = 'formatdate("HH \'Hours and \'M \'Minute(s)\'", "2018-01-02T23:12:01Z")'
        expected = "11 Hours and 1 Minute(s)"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_handle_for_loop_in_dict(self):
        input_str = "{for val in [{'name': 'key3'},{'name': 'key4'}] : val.name => true}"
        expected = {'key3': 'true', 'key4': 'true'}
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_handle_for_loop_in_list(self):
        input_str = "[for val in ['k', 'v'] : val]"
        expected = ['k', 'v']
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = "{for val in ['k', 'v'] : val.name => true}"
        expected = "{for val in ['k', 'v'] : val.name :> true}"
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_handle_for_loop_in_list_of_dicts(self):
        input_str = "[for val in [{'name': 'raw', 'type': 'container'}, {'name': 'masked', 'type': 'blob'}] : {'name': '${val.name}', 'type': '${val.type}'}]"
        expected = [{'name': 'raw', 'type': 'container'}, {'name': 'masked', 'type': 'blob'}]
        self.assertEqual(expected, evaluate_terraform(input_str))

        input_str = "[for val in [{'a': 123, 'b': True, 'c': None}] : {'a': '${val.a}', 'b': '${val.b}', 'c': '${val.c}'}]"
        expected = [{'a': 123, 'b': True, 'c': None}]
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_base64_value(self):
        input_str = "\"['dGVzdA==']\""
        expected = ["dGVzdA=="]
        self.assertEqual(expected, evaluate_terraform(input_str))

    def test_try_block(self):
        input_str = 'try("local.foo.boop", "{}")'
        expected = {}
        result = evaluate_terraform(input_str)
        self.assertEqual(expected, result)

    def test_try_then_merge_block(self):
        input_str = "try((merge({}, {})), 1, 2)"
        expected = {}
        result = evaluate_terraform(input_str)
        self.assertEqual(expected, result)

    def test_empty_string(self):
        input_str = "   "
        expected = input_str
        result = evaluate_terraform(input_str)
        self.assertEqual(expected, result)

        input_str = ""
        expected = input_str
        result = evaluate_terraform(input_str)
        self.assertEqual(expected, result)

    def test_dict_as_string(self):
        expected = {'Statement': [
            {'Action': ['lambda:CreateFunction', 'lambda:CreateEventSourceMapping', 'dynamodb:CreateTable'],
             'Effect': 'Allow', 'Resource': '*'}], 'Version': '2012-10-17'}
        input_str = '  {    "Version": "2012-10-17",    "Statement": [      {        "Effect": "Allow",        "Action": [          "lambda:CreateFunction",          "lambda:CreateEventSourceMapping",          "dynamodb:CreateTable",        ],        "Resource": "*"      }    ]  }'
        result = evaluate_terraform(input_str)
        assert result == expected

    def test_continue_stays_the_same(self):
        expected = "continue"
        result = evaluate_terraform("continue")
        self.assertEqual(expected, result)


@pytest.mark.parametrize(
    "origin_str,str_to_replace,new_value,expected",
    [
        ("${lookup({'a': ${local.protocol1}},\"a\",\"https\")}", '${local.protocol1}', 'local.protocol1', '"local.protocol1"'),
        ('${length(keys(var.identity)) > 0 ? [${var.identity}] : []}', '${var.identity}', 'var.identity', 'var.identity'),
    ],
    ids=["escaped", "not escaped"],
)
def test_find_new_value_for_interpolation(origin_str: str, str_to_replace: str, new_value: str, expected: str):
    actual = _find_new_value_for_interpolation(origin_str, str_to_replace, new_value)
    assert actual == expected


def test_evaluate_range_pattern() -> None:

    # Test range pattern
    assert evaluate("1-10") == "1-10"
    assert evaluate("5-25")== "5-25"
    assert evaluate("10-5") == 5

    # Test non-range pattern for comparison
    assert evaluate("1+1") == 2


EVAL_DANGEROUS_INPUTS = [
    ("unicode_characters", "[{}._﹎class_﹎._﹎bases_﹎[0]._﹎subclasses_﹎()]"),
    ("generators_to_bypass_protections_code", "((gen:=(gen_list[0].gi_frame.f_back.f_back.f_globals for _ in [1,]),v:=gen_list.append(gen), gen_list[0])[2] for gen_list in [[]]).send(None).send(None).get('_'+'_builtins_'+'_')['_'+'_import_'+'_']('os').system('date >> /tmp/gen-example')"),
    ("system command", "__import__('os').system('rm -rf /')"),
    ("read passwd", "open('/etc/passwd').read()"),
    ("access builtins", "__builtins__.__import__('os').listdir('.')"),
    ("exec call", "exec('print(123)')"),
    ("create file", "__import__('os').popen('touch /tmp/hacked').read()"),
    ("subclasses", "().__class__.__bases__[0].__subclasses__()"),
    ("socket connection", "__import__('socket').socket().connect(('example.com', 80))"),
    ("lambda trick", "(lambda f: f('ls'))(__import__('os').system)")
]


@pytest.mark.parametrize("description, input_str", EVAL_DANGEROUS_INPUTS)
def test_evaluate_malicious_code(description: str, input_str: str)-> None:
    expected = input_str
    result = evaluate_terraform(input_str)
    assert result == expected
    asteval = get_asteval()
    asteval(input_str)
    assert asteval.error


import unittest

import hcl2

from checkov.terraform.checks.resource.azure.DataFactoryNoPublicNetworkAccess import check
from checkov.common.models.enums import CheckResult


class TestDataFactoryNoPublicNetworkAccess(unittest.TestCase):

    def test_failure_missing_attribute(self):
        hcl_res = hcl2.loads("""
          resource "azurerm_data_factory" "example" {
              name                = "example"
              location            = "azurerm_resource_group.example.location"
              resource_group_name = "azurerm_resource_group.example.name"
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_data_factory']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_data_factory" "example" {
              name                = "example"
              location            = "azurerm_resource_group.example.location"
              resource_group_name = "azurerm_resource_group.example.name"
              public_network_enabled = true
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_data_factory']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_data_factory" "example" {
              name                = "example"
              location            = "azurerm_resource_group.example.location"
              resource_group_name = "azurerm_resource_group.example.name"
              public_network_enabled = false
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_data_factory']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()


from checkov.terraform_json.parser import hclify, prepare_definition


def test_hclify():
    # given
    bucket_version = {
        "//": {
            "metadata": {
                "path": "AppStack/bucket_version",
                "uniqueId": "bucket_version",
            }
        },
        "bucket": "${aws_s3_bucket.bucket.bucket}",
        "versioning_configuration": {
            "status": "Enabled",
        },
    }

    # when
    result = hclify(obj=bucket_version)

    # then
    assert result == {
        "//": {
            "metadata": {
                "path": "AppStack/bucket_version",
                "uniqueId": "bucket_version",
            }
        },
        "bucket": ["${aws_s3_bucket.bucket.bucket}"],
        "versioning_configuration": [
            {
                "status": ["Enabled"],
            }
        ],
    }


def test_prepare_definition_locals():
    cdk_definition = {
        "locals": {
            "bucket_name": "example",
            "http_endpoint": "disabled",
            "__startline__": 1,
            "__endline__": 2,
        }
    }

    # when
    tf_definition = prepare_definition(cdk_definition)

    # then
    assert tf_definition == {
        "locals": [
            {
                "bucket_name": ["example"],
                "http_endpoint": ["disabled"],
                "__startline__": 1,
                "__endline__": 2,
            }
        ]
    }



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.MySQLPublicAccessDisabled import check
from checkov.common.models.enums import CheckResult


class TestMySQLPublicAccessDisabled(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mysql_server" "examplea" {
  name                = var.mysqlserver_name
  location            = var.resource_group.location
  resource_group_name = var.resource_group.name

  administrator_login          = var.admin_name
  administrator_login_password = var.password
  sku_name = var.sku_name
  storage_mb = var.storage_mb
  version    = var.server_version

  auto_grow_enabled            = true
  backup_retention_days        = 7
  geo_redundant_backup_enabled = false
  infrastructure_encryption_enabled = false
    public_network_access_enabled = true
}
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mysql_server']['examplea']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_missing_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mysql_server" "examplea" {
  name                = var.mysqlserver_name
  location            = var.resource_group.location
  resource_group_name = var.resource_group.name

  administrator_login          = var.admin_name
  administrator_login_password = var.password
  sku_name = var.sku_name
  storage_mb = var.storage_mb
  version    = var.server_version

  auto_grow_enabled            = true
  backup_retention_days        = 7
  geo_redundant_backup_enabled = false
  infrastructure_encryption_enabled = false
}
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mysql_server']['examplea']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)
        
    def test_success(self):
        hcl_res = hcl2.loads("""
resource "azurerm_mysql_server" "examplea" {
  name                = var.mysqlserver_name
  location            = var.resource_group.location
  resource_group_name = var.resource_group.name

  administrator_login          = var.admin_name
  administrator_login_password = var.password
  sku_name = var.sku_name
  storage_mb = var.storage_mb
  version    = var.server_version

  auto_grow_enabled            = true
  backup_retention_days        = 7
  geo_redundant_backup_enabled = false
  infrastructure_encryption_enabled = false
  public_network_access_enabled = false
}
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mysql_server']['examplea']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from pathlib import Path

from parameterized import parameterized_class

from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = Path(__file__).parent


@parameterized_class([{"graph_framework": "NETWORKX"}, {"graph_framework": "IGRAPH"}])
class ExistsSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = str(TEST_DIRNAME)
        super().setUp()

    def test_allow_list(self):
        # given
        root_folder = TEST_DIRNAME.parents[2] / "resources/encryption_test"
        check_id = "ResourceAllowList"
        should_pass = [
            "aws_s3_bucket.encrypted_bucket",
            "aws_s3_bucket.unencrypted_bucket",
        ]
        should_fail = [
            "aws_rds_cluster.rds_cluster_encrypted",
            "aws_rds_cluster.rds_cluster_unencrypted",
            "aws_neptune_cluster.encrypted_neptune",
            "aws_neptune_cluster.unencrypted_neptune",
        ]
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        # when/then
        self.run_test(root_folder=str(root_folder), expected_results=expected_results, check_id=check_id)



import unittest

from checkov.terraform.checks.resource.gcp.GoogleCloudSqlDatabaseRequireSsl import check
from checkov.common.models.enums import CheckResult


class GoogleCloudSqlDatabaseRequireSsl(unittest.TestCase):

    def test_failure(self):
        resource_conf = {'name': ['google_cluster'], 'monitoring_service': ['none']}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_requiressl_false(self):
        resource_conf = {'settings': [{'tier': ['1'], 'ip_configuration': [{'require_ssl': [False]}]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_requiressl(self):
        resource_conf = {'settings': [{'tier': ['1'], 'ip_configuration': [{'require_ssl': [True]}]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure_sslmode_encryptonly(self):
        resource_conf = {'settings': [{'tier': ['1'], 'ip_configuration': [{'ssl_mode': ["ENCRYPTED_ONLY"]}]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_sslmode_trustedclient(self):
        resource_conf = {'settings': [{'tier': ['1'], 'ip_configuration': [{'ssl_mode': ["TRUSTED_CLIENT_CERTIFICATE_REQUIRED"]}]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_sqlserver_sslmode_encryptedonly(self):
        resource_conf = {'database_version': ["SQLSERVER_2022_STANDARD"],'settings': [{'tier': ['1'], 'ip_configuration': [{'ssl_mode': ["ENCRYPTED_ONLY"]}]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

if __name__ == '__main__':
    unittest.main()



"""Behavioural tests for the external-checks verification library."""
from __future__ import annotations

import os
from pathlib import Path

import pytest

from checkov.common.external_checks.verification import (
    SignatureVerificationError,
    VerificationResult,
    load_public_keys,
    verify_bytes,
    verify_external_checks_dirs,
    verify_file,
)


def test_accepts_valid_input(
    valid_dir: Path, valid_bodies: "dict[str, bytes]",
    key_a_pub_pem: bytes, tmp_path: Path,
):
    """A signed directory verifies and returns the trailer-stripped bytes."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    verified = verify_external_checks_dirs([str(valid_dir)], keys)

    assert sorted(os.path.basename(p) for p in verified) == sorted(
        ["__init__.py", "_helper.py", "aws_check.py"]
    )
    # Returned bytes are the trailer-stripped bodies (what the loader
    # will exec), not the bytes on disk (which include the trailer line).
    for path, returned_bytes in verified.items():
        name = os.path.basename(path)
        assert returned_bytes == valid_bodies[name], f"byte mismatch for {name}"

    helper_bytes = verified[str(valid_dir / "_helper.py")]
    assert len(helper_bytes) > 1024  # nontrivial size catches truncation bugs


def test_accepts_with_any_configured_key(
    valid_dir: Path, key_a_pub_pem: bytes, key_b_pub_pem: bytes, tmp_path: Path,
):
    """Rotation: file signed by key A passes when verifier has [B, A] configured."""
    key_a = tmp_path / "key_a.pem"
    key_a.write_bytes(key_a_pub_pem)
    key_b = tmp_path / "key_b.pem"
    key_b.write_bytes(key_b_pub_pem)

    keys = load_public_keys([str(key_b), str(key_a)])
    verified = verify_external_checks_dirs([str(valid_dir)], keys)
    assert len(verified) == 3


def test_rejects_invalid_input(
    mutated_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """Bytes appended after the trailer → directory is rejected.

    Either ``signature verification failed`` (BAD_SIG/UNKNOWN_KEY) or
    ``missing signature`` (NO_SIG) is acceptable — both are structurally
    correct refusals of a file with extra bytes after the trailer's
    newline.
    """
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs([str(mutated_dir)], keys)

    msg = str(exc.value).lower()
    assert "signature verification failed" in msg or "missing signature" in msg


def test_rejects_missing_signature(
    unsigned_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """No trailer line → ``missing signature``."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs([str(unsigned_dir)], keys)

    assert "missing signature" in str(exc.value).lower()


def test_rejects_unknown_key(
    valid_dir: Path, key_b_pub_pem: bytes, tmp_path: Path,
):
    """Signed by a key not in the trust list → rejected."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_b_pub_pem)  # B configured, dir signed by A
    keys = load_public_keys([str(key_path)])

    with pytest.raises(SignatureVerificationError):
        verify_external_checks_dirs([str(valid_dir)], keys)


def test_rejects_directory_with_partial_signing(
    partial_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """Whole directory is rejected if any Python file lacks a trailer."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs([str(partial_dir)], keys)

    assert "__init__.py" in str(exc.value)


def test_ignores_non_python_text_files(
    mixed_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """Non-Python text files (YAML / Markdown / LICENSE) are silently passed.

    Binary loadable files are different — they are hard-rejected (see
    ``test_rejects_dir_containing_binary_loadable_file``).
    """
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    verified = verify_external_checks_dirs([str(mixed_dir)], keys)
    assert not any(p.endswith((".yaml", ".md", "LICENSE")) for p in verified)
    assert any(p.endswith("aws_check.py") for p in verified)


def test_is_noop_without_configured_keys(valid_dir: Path):
    """Calling the verifier with no keys is a no-op."""
    verified = verify_external_checks_dirs([str(valid_dir)], [])
    assert verified == {}


def test_reports_offending_path_on_failure(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """A single bad file in a directory of many produces an error that names it."""
    (valid_dir / "_helper.py").write_bytes(b"# modified\nHELPER = 'changed'\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs([str(valid_dir)], keys)
    assert "_helper.py" in str(exc.value)


@pytest.mark.parametrize("format_id", ["format_a", "format_b", "format_c", "format_d", "format_e"])
def test_rejects_unsupported_key_format(
    format_id: str,
    unsupported_key_format_a: bytes,
    unsupported_key_format_b: bytes,
    unsupported_key_format_c: bytes,
    unsupported_key_format_d: bytes,
    unsupported_key_format_e: bytes,
    tmp_path: Path,
):
    """Public keys that aren't P-256 ECDSA are rejected up-front.

    Covers RSA (format_a), Ed25519 (format_b), P-384 (format_c),
    SECP256K1 (format_d — the Bitcoin curve, which operators are most
    likely to accidentally use because it shows up in countless ECDSA
    tutorials and Stack Overflow answers), and P-521 (format_e — a NIST
    EC curve that looks superficially like P-256 but has a different
    key size).
    """
    pem_by_id = {
        "format_a": unsupported_key_format_a,
        "format_b": unsupported_key_format_b,
        "format_c": unsupported_key_format_c,
        "format_d": unsupported_key_format_d,
        "format_e": unsupported_key_format_e,
    }
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(pem_by_id[format_id])

    with pytest.raises(SignatureVerificationError) as exc:
        load_public_keys([str(key_path)])

    assert "unsupported key format" in str(exc.value).lower()


def test_rejects_malformed_signature(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """A trailer whose hex decodes to non-DER bytes is rejected.

    71 bytes of \\x00 satisfies the length range but fails strict-DER
    decode → BAD_SIGNATURE.
    """
    target = valid_dir / "aws_check.py"
    body = b"# valid check\nCHECK_ID = 'CKV_T_1'\n"
    bad_payload = (b"\x00" * 71).hex().encode("ascii")
    target.write_bytes(body + b"# checkov-digest: " + bad_payload + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs([str(valid_dir)], keys)
    assert "signature verification failed" in str(exc.value).lower()


def test_rejects_path_escape_via_link(
    link_escape_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """A symlink that resolves outside the verified directory is rejected."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    with pytest.raises(SignatureVerificationError):
        verify_external_checks_dirs([str(link_escape_dir)], keys)


def test_verify_file_returns_ok_for_valid_input(
    valid_dir: Path, valid_bodies: "dict[str, bytes]",
    key_a_pub_pem: bytes, tmp_path: Path,
):
    """``verify_file(path, keys)`` returns ``(OK, signed_bytes)`` for a signed file."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, signed_bytes = verify_file(str(valid_dir / "aws_check.py"), keys)
    assert result == VerificationResult.OK
    assert signed_bytes == valid_bodies["aws_check.py"]


def test_verify_file_returns_no_signature_when_trailer_absent(
    unsigned_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """File without trailer line → ``(NO_SIGNATURE, b"")``."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, signed_bytes = verify_file(str(unsigned_dir / "aws_check.py"), keys)
    assert result == VerificationResult.NO_SIGNATURE
    assert signed_bytes == b""


def test_verify_file_returns_unknown_key_when_bytes_changed(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """Replacing the body but keeping the (now stale) trailer → UNKNOWN_KEY.

    The trailer is still a well-formed DER ECDSA pair; it just no longer
    verifies for the new body. UNKNOWN_KEY, not BAD_SIGNATURE.
    """
    target = valid_dir / "aws_check.py"
    original = target.read_bytes()
    body = original[:-1]  # strip terminating \n
    last_nl = body.rfind(b"\n")
    trailer_with_nl = original[last_nl + 1:]  # trailer line + \n
    target.write_bytes(b"# modified body\n" + trailer_with_nl)

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, signed_bytes = verify_file(str(target), keys)
    assert result == VerificationResult.UNKNOWN_KEY
    assert signed_bytes == b""


def test_verify_file_returns_io_error_when_file_unreadable(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """File that cannot be read returns ``(IO_ERROR, b"")``.

    We open a directory path: ``open(directory, "rb")`` raises
    ``IsADirectoryError`` on POSIX, ``PermissionError`` on Windows —
    both ``OSError`` subclasses the verifier catches.
    """
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, signed_bytes = verify_file(str(valid_dir), keys)
    assert result == VerificationResult.IO_ERROR
    assert signed_bytes == b""


def test_rejects_nonexistent_dir(key_a_pub_pem: bytes, tmp_path: Path):
    """A typo'd directory path raises, not silently returns an empty map."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs([str(tmp_path / "does-not-exist")], keys)
    assert "does not exist" in str(exc.value)


def test_empty_string_dir_entries_are_ignored(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """Empty-string entries in dirs[] are silently skipped (CLI artefact)."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])
    verified = verify_external_checks_dirs(["", str(valid_dir), ""], keys)
    assert len(verified) == 3


def test_path_escape_does_not_mask_other_failures_in_same_dir(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """A symlink-escape and a missing-trailer in the same dir are both reported."""
    root = tmp_path / "mixed"
    root.mkdir()
    outside = tmp_path / "outside"
    outside.mkdir()
    (outside / "outside_module.py").write_bytes(b"# outside")
    try:
        os.symlink(outside / "outside_module.py", root / "linked.py")
    except (OSError, NotImplementedError):
        pytest.skip("symlinks unsupported on this platform")
    (root / "unsigned.py").write_bytes(b"# no trailer\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs([str(root)], keys)
    msg = str(exc.value)
    assert "linked.py" in msg
    assert "unsigned.py" in msg


def test_rejects_duplicate_dirs(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])
    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs([str(valid_dir), str(valid_dir)], keys)
    assert "duplicate" in str(exc.value).lower()


def test_rejects_nested_dirs(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """A parent dir and one of its children cannot both be configured."""
    parent = valid_dir.parent
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])
    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs([str(parent), str(valid_dir)], keys)
    assert "overlap" in str(exc.value).lower()


def test_duplicate_and_nonexistent_dirs_are_both_reported(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """A typo'd dir and a duplicate dir in the same call must both surface."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    typo = str(tmp_path / "does-not-exist")
    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs(
            [str(valid_dir), str(valid_dir), typo], keys,
        )
    msg = str(exc.value)
    assert "duplicate" in msg.lower(), "duplicate dir not reported"
    assert "does not exist" in msg, "typo'd dir not reported"


def test_duplicate_dir_does_not_skip_other_dirs(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path,
):
    """When dirs=[A, A, B] and B has a missing-trailer file, B's failure must
    still be reported alongside the duplicate-A error."""
    second = tmp_path / "second"
    second.mkdir()
    (second / "unsigned.py").write_bytes(b"# no trailer\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs(
            [str(valid_dir), str(valid_dir), str(second)], keys,
        )
    msg = str(exc.value)
    assert "duplicate" in msg.lower()
    assert "unsigned.py" in msg


def test_verify_bytes_rejects_empty_key_list():
    """``verify_bytes`` requires at least one key; empty list raises ValueError."""
    with pytest.raises(ValueError):
        verify_bytes(b"# checkov-digest: 00\n", [])


def test_sibling_prefix_collision_is_not_an_escape(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes, make_trailer,
):
    """``/x/checks-other/foo.py`` must not be considered inside ``/x/checks``."""
    checks = tmp_path / "checks"
    checks.mkdir()
    (checks / "foo.py").write_bytes(make_trailer(b"# inside\n", priv_a))

    checks_other = tmp_path / "checks-other"
    checks_other.mkdir()
    (checks_other / "bad.py").write_bytes(b"# outside\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    verified = verify_external_checks_dirs([str(checks)], keys)
    assert set(verified) == {str(checks / "foo.py")}


def test_dangling_symlink_is_rejected(
    tmp_path: Path, key_a_pub_pem: bytes,
):
    """A .py symlink whose target does not exist is rejected."""
    root = tmp_path / "dangling"
    root.mkdir()
    try:
        os.symlink(tmp_path / "no-such-target", root / "broken.py")
    except (OSError, NotImplementedError):
        pytest.skip("symlinks unsupported on this platform")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])
    with pytest.raises(SignatureVerificationError) as exc:
        verify_external_checks_dirs([str(root)], keys)
    assert "broken.py" in str(exc.value)


# Trailer-shape / wire-format rules.

def test_rejects_file_without_trailing_newline(
    valid_dir: Path, priv_a, key_a_pub_pem: bytes, tmp_path: Path,
):
    """A file that does not end with \\n cannot carry a trailer → NO_SIGNATURE."""
    target = valid_dir / "aws_check.py"
    body = target.read_bytes()
    assert body.endswith(b"\n")
    target.write_bytes(body.rstrip(b"\n"))

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, signed_bytes = verify_file(str(target), keys)
    assert result == VerificationResult.NO_SIGNATURE
    assert signed_bytes == b""


def test_rejects_file_with_double_trailing_newline(
    valid_dir: Path, priv_a, key_a_pub_pem: bytes, tmp_path: Path,
):
    """A file ending in \\n\\n — the "last line" is empty → NO_SIGNATURE.

    Most-likely IDE format-on-save failure (an end-of-file-fixer hook
    adding a second \\n).
    """
    target = valid_dir / "aws_check.py"
    target.write_bytes(target.read_bytes() + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, _ = verify_file(str(target), keys)
    assert result == VerificationResult.NO_SIGNATURE


def test_rejects_crlf_terminated_file(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes, make_trailer,
):
    """A file with CRLF line endings → BAD_SIGNATURE.

    The last byte is \\n; the trailer's terminator was rewritten as
    \\r\\n. The trailer prefix slice still starts cleanly with
    ``# checkov-digest: ``, but the trailing \\r ends up inside the hex
    payload — \\r is not lowercase hex, so the payload guard rejects
    the trailer. The prefix-was-present check then maps the failure to
    BAD_SIGNATURE (corrupt trailer), not NO_SIGNATURE (no trailer at all).
    """
    body = b"def check():\r\n    return 'ok'\r\n"
    signed = make_trailer(body, priv_a)
    crlf_signed = signed[:-1] + b"\r\n"

    target = tmp_path / "check.py"
    target.write_bytes(crlf_signed)

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, _ = verify_file(str(target), keys)
    assert result == VerificationResult.BAD_SIGNATURE


def test_rejects_file_with_bom(
    valid_dir: Path, priv_a, key_a_pub_pem: bytes, tmp_path: Path,
):
    """A BOM prepended to a signed file → UNKNOWN_KEY.

    The signer signed bytes without a BOM. The verifier hashes the
    on-disk bytes including the BOM → mismatch. The trailer is
    structurally fine, so we land in UNKNOWN_KEY rather than
    NO_SIGNATURE / BAD_SIGNATURE.
    """
    target = valid_dir / "aws_check.py"
    target.write_bytes(b"\xef\xbb\xbf" + target.read_bytes())

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, _ = verify_file(str(target), keys)
    assert result == VerificationResult.UNKNOWN_KEY


@pytest.mark.parametrize(
    ("mutation_id", "trailer_replacement_bytes", "expected_result"),
    [
        # No space at all → the prefix ``# checkov-digest: `` does not
        # match → no trailer detected at all → NO_SIGNATURE.
        ("no_space_after_colon", b"# checkov-digest:", VerificationResult.NO_SIGNATURE),
        ("tab_after_colon", b"# checkov-digest:\t", VerificationResult.NO_SIGNATURE),
        # Extra space after the required one → prefix DOES match (prefix
        # is "# checkov-digest: ", and ":  " starts with ": "), but the
        # leading space ends up inside the hex payload and fails the
        # alphabet guard → BAD_SIGNATURE.
        ("two_spaces_after_colon", b"# checkov-digest:  ", VerificationResult.BAD_SIGNATURE),
        # Uppercase hex → prefix matches, payload fails the lowercase
        # alphabet guard → BAD_SIGNATURE.
        ("uppercase_hex_prefix", b"# checkov-digest: ", VerificationResult.BAD_SIGNATURE),
    ],
)
def test_rejects_trailer_with_wrong_whitespace_or_case(
    mutation_id: str,
    trailer_replacement_bytes: bytes,
    expected_result: VerificationResult,
    tmp_path: Path,
    priv_a,
    key_a_pub_pem: bytes,
    make_sign,
):
    """Trailer-shape mutations are rejected.

    Two-tier result mapping: if the prefix doesn't even match → the
    customer gets ``NO_SIGNATURE`` ("you forgot to sign"). If the prefix
    matches but the payload is structurally wrong → ``BAD_SIGNATURE``
    ("you signed it wrong"). Both diagnostics are useful to the customer
    in different ways.
    """
    body = b"def x():\n    pass\n"
    sig_der = make_sign(priv_a, body)
    hex_payload = sig_der.hex()
    if mutation_id == "uppercase_hex_prefix":
        hex_payload = hex_payload.upper()
    file_bytes = body + trailer_replacement_bytes + hex_payload.encode("ascii") + b"\n"

    target = tmp_path / "check.py"
    target.write_bytes(file_bytes)

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, _ = verify_file(str(target), keys)
    assert result == expected_result


def test_rejects_trailer_with_huge_hex_payload(
    tmp_path: Path, key_a_pub_pem: bytes,
):
    """A trailer-prefixed line with a 100 KiB hex payload → BAD_SIGNATURE.

    The parser's length bounds reject the trailer before any crypto, but
    the prefix is still present → BAD_SIGNATURE (not NO_SIGNATURE),
    distinguishing "trailer attempted but malformed" from "no trailer".
    """
    body = b"def x():\n    pass\n"
    huge_hex = b"ab" * 51200  # 100 KiB of valid lowercase hex
    file_bytes = body + b"# checkov-digest: " + huge_hex + b"\n"

    target = tmp_path / "check.py"
    target.write_bytes(file_bytes)

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, _ = verify_file(str(target), keys)
    assert result == VerificationResult.BAD_SIGNATURE


@pytest.mark.parametrize("extension", [".pyc", ".so", ".pyd", ".pyi"])
def test_binary_loadable_files_are_silently_ignored(
    extension: str, binary_loadable_dir: Path,
    key_a_pub_pem: bytes, tmp_path: Path,
):
    """Binary loadable files inside an external-checks dir are silently ignored.

    Only ``.py`` files are imported via the external-checks loader, so
    other file types (``.pyc`` / ``.so`` / ``.pyd`` / ``.pyi``) are out
    of scope for trailer signing. They must not cause the scan to fail.
    """
    (binary_loadable_dir / f"native{extension}").write_bytes(b"\x00\x01\x02")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    # Must not raise — binary loadables alongside a valid signed .py are ignored.
    verified = verify_external_checks_dirs([str(binary_loadable_dir)], keys)
    assert any(p.endswith("aws_check.py") for p in verified)
    assert not any(p.endswith(extension) for p in verified)


def test_accepts_empty_init_py_signed_as_comment_only(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes, make_trailer,
):
    """A signed-empty ``__init__.py`` verifies with signed_bytes == b"".

    Empty init files become exactly ``# checkov-digest: <hex>\\n`` — the
    trailer line is the entire file. The signed bytes are b"", and
    Python's package-init contract is satisfied (existence is what
    ``importlib`` checks, not contents).
    """
    root = tmp_path / "pkg"
    root.mkdir()
    init_path = root / "__init__.py"
    init_path.write_bytes(make_trailer(b"", priv_a))

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, signed_bytes = verify_file(str(init_path), keys)
    assert result == VerificationResult.OK
    assert signed_bytes == b""

    verified = verify_external_checks_dirs([str(root)], keys)
    assert verified == {str(init_path): b""}


def test_init_py_signed_as_empty_loads_as_empty_package(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes, make_trailer,
):
    """The b"" returned by the verifier compiles and execs into an empty module.

    Proves the loader will get a working module object when it runs
    ``exec(compile(signed_bytes, path, "exec"), ...)`` against the empty
    init file.
    """
    init_path = tmp_path / "__init__.py"
    init_path.write_bytes(make_trailer(b"", priv_a))

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, signed_bytes = verify_file(str(init_path), keys)
    assert result == VerificationResult.OK

    code = compile(signed_bytes, str(init_path), "exec")
    namespace: dict = {}
    exec(code, namespace)  # noqa: S102 — intentional, this is the loader's contract
    user_keys = [k for k in namespace if not k.startswith("__")]
    assert user_keys == []


def test_ignores_extra_trailer_line_in_middle_of_file(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes, make_trailer,
):
    """A bogus ``# checkov-digest:`` line in the middle of the file is ignored.

    Only the last line is checked. The real trailer covers the whole
    body — including any earlier digest-looking line inside it.
    """
    body = (
        b"# example trailer below for documentation:\n"
        b"# checkov-digest: deadbeef\n"
        b"def check():\n"
        b"    return 'ok'\n"
    )
    target = tmp_path / "check.py"
    target.write_bytes(make_trailer(body, priv_a))

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, signed_bytes = verify_file(str(target), keys)
    assert result == VerificationResult.OK
    assert signed_bytes == body


def test_ignores_length_valid_fake_trailer_inside_signed_body(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes, make_trailer,
):
    """A LENGTH-VALID fake ``# checkov-digest:`` mid-body still verifies OK.

    Sibling to ``test_ignores_extra_trailer_line_in_middle_of_file``,
    which uses ``deadbeef`` (8 hex chars). That length is rejected by
    the parser's ``_HEX_MIN <= len <= _HEX_MAX`` (126-144) guard
    BEFORE the "only the last line counts" invariant is even
    exercised — so the existing test would pass even with a buggy
    parser that scanned every line.

    This sibling uses a 128-hex-char imposter (valid length, just not
    the real signature), proving the "last line only" rule alone
    catches the imposter even if the length filter accidentally
    accepted it. Pins the parser invariant at the right scope.
    """
    fake_trailer_line = b"# checkov-digest: " + b"ab" * 64 + b"\n"
    assert 126 <= len(b"ab" * 64) <= 144, (
        "precondition: the imposter trailer must be length-valid "
        "(otherwise we're testing the same path as the sibling test)"
    )

    body = (
        b"# real check\n"
        + fake_trailer_line  # imposter, mid-body
        + b"def check():\n    return 'ok'\n"  # real code AFTER the imposter
    )

    target = tmp_path / "imposter.py"
    target.write_bytes(make_trailer(body, priv_a))

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, signed_bytes = verify_file(str(target), keys)
    assert result == VerificationResult.OK, (
        f"length-valid mid-body fake trailer wrongly altered the result "
        f"to {result!r}; the 'last line only' rule should hold "
        f"regardless of imposter length"
    )
    assert signed_bytes == body, (
        "the verifier returned a different body than the original — the "
        "fake trailer was treated as a structural element, not data"
    )


def test_rejects_trailer_with_appended_garbage(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes, make_trailer,
):
    """Junk bytes appended after the trailer's \\n → NO_SIGNATURE."""
    body = b"def check():\n    return 'ok'\n"
    target = tmp_path / "check.py"
    target.write_bytes(make_trailer(body, priv_a) + b"JUNK")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, _ = verify_file(str(target), keys)
    assert result == VerificationResult.NO_SIGNATURE


@pytest.mark.parametrize(
    "mutation_id",
    [
        "strip_trailing_newline",
        "add_second_newline",
        "normalise_to_crlf",
    ],
)
def test_rejects_formatter_simulated_trailer_mutation(
    mutation_id: str,
    tmp_path: Path,
    priv_a,
    key_a_pub_pem: bytes,
    make_trailer,
):
    """Format-on-save style mutations all fail with a non-OK result.

    Pins the scenario where an IDE format-on-save hook corrupts the
    trailer. We assert only that the result is non-OK — the wire-format
    rules return a non-OK result in several ways for these mutations,
    and pinning the exact result per mutation would over-fit the test
    to parser internals.

    If any mutation ever returns OK silently, customers' signed files
    would start failing verification randomly in CI; this is the canary
    test for that regression.
    """
    body = b"def check():\n    return 'ok'\n"
    signed = make_trailer(body, priv_a)

    if mutation_id == "strip_trailing_newline":
        mutated = signed.rstrip(b"\n")
    elif mutation_id == "add_second_newline":
        mutated = signed + b"\n"
    elif mutation_id == "normalise_to_crlf":
        mutated = signed.replace(b"\n", b"\r\n")
    else:
        raise AssertionError(f"unknown mutation_id {mutation_id}")

    target = tmp_path / "check.py"
    target.write_bytes(mutated)

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, signed_bytes = verify_file(str(target), keys)
    assert result != VerificationResult.OK, (
        f"mutation {mutation_id!r} was silently accepted"
    )
    assert signed_bytes == b""



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.terraform.plan_parser import TF_PLAN_RESOURCE_CHANGE_ACTIONS


class KmsKeyNotDeleted(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Secret is not deleted"
        id = "CUSTOM_DELETE_1"
        supported_resources = ("aws_secretsmanager_secret",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        actions = conf.get(TF_PLAN_RESOURCE_CHANGE_ACTIONS)
        if isinstance(actions, list) and "delete" in actions:
            self.details.append("some great details")
            return CheckResult.FAILED
        return CheckResult.PASSED


scanner = KmsKeyNotDeleted()



from aws_cdk import core
from aws_cdk import aws_lambda as _lambda
from aws_cdk import aws_ec2 as ec2
from aws_cdk import aws_sam as sam

class MyLambdaStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a Lambda function in the VPC
        my_lambda_function = _lambda.Function(
            self,
            "MyLambdaFunction",
            runtime=_lambda.Runtime.PYTHON_3_8,
            handler="index.handler",
            code=_lambda.Code.from_asset("path/to/your/code"),
            function_name="my-function-name",
            security_group=my_vpc.vpc_default_security_group,
            allow_public_subnet=False,  # Set to True if you want public subnets
        )

class MySAMLambdaStack2(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a custom VPC
        my_vpc = ec2.Vpc(
            self,
            "MyVPC",
            max_azs=2,  # Set the number of Availability Zones as needed
        )

        # Define the Serverless::Function within the VPC
        my_sam_lambda_function = sam.CfnFunction(
            self,
            "MySAMLambdaFunction",
            handler="index.handler",
            runtime="nodejs14.x",
            code_uri="./my-code",
        )




import os
import unittest
from xml.dom import minidom

current_dir = os.path.dirname(os.path.realpath(__file__))


class TestCheckovCyclonedxReport(unittest.TestCase):
    def test_terragoat_report(self):
        report_path = os.path.join(os.path.dirname(current_dir), "checkov_report_terragoat_cyclonedx.xml")
        self.validate_report(os.path.abspath(report_path))

    def validate_report(self, report_path: str) -> None:
        with open(report_path) as cyclonedx_file:
            data = minidom.parse(cyclonedx_file)
            self.validate_report_not_empty(data)

    def validate_report_not_empty(self, report):
        vulnrability_file = (
            report.getElementsByTagNameNS("*", "vulnerabilities")[0]
            .getElementsByTagNameNS("*", "vulnerability")[0]
            .getElementsByTagNameNS("*", "id")[0]
            .firstChild.nodeValue
        )
        self.assertTrue(vulnrability_file.startswith("CKV"))


if __name__ == "__main__":
    unittest.main()



import unittest

from checkov.terraform.checks.resource.aws.DynamodbRecovery import check
from checkov.common.models.enums import CheckResult


class TestDynamodbRecovery(unittest.TestCase):
    def test_failure(self):
        resource_conf = {
            "name": ["violations_for_resources${var.unique_tag}"],
            "billing_mode": ["PAY_PER_REQUEST"],
            "hash_key": ["id"],
            "range_key": ["violation_id"],
            "local_secondary_index": [
                {"name": ["violation_id_index"], "projection_type": ["ALL"], "range_key": ["violation_id"]}
            ],
            "global_secondary_index": [
                {
                    "hash_key": ["violation_id"],
                    "name": ["violation_id-aws_account_id-index"],
                    "range_key": ["aws_account_id"],
                    "projection_type": ["ALL"],
                }
            ],
            "attribute": [
                {"name": ["id"], "type": ["S"]},
                {"name": ["violation_id"], "type": ["S"]},
                {"name": ["aws_account_id"], "type": ["S"]},
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {
            "name": ["violations_for_resources${var.unique_tag}"],
            "billing_mode": ["PAY_PER_REQUEST"],
            "hash_key": ["id"],
            "range_key": ["violation_id"],
            "local_secondary_index": [
                {"name": ["violation_id_index"], "projection_type": ["ALL"], "range_key": ["violation_id"]}
            ],
            "global_secondary_index": [
                {
                    "hash_key": ["violation_id"],
                    "name": ["violation_id-aws_account_id-index"],
                    "range_key": ["aws_account_id"],
                    "projection_type": ["ALL"],
                }
            ],
            "attribute": [
                {"name": ["id"], "type": ["S"]},
                {"name": ["violation_id"], "type": ["S"]},
                {"name": ["aws_account_id"], "type": ["S"]},
            ],
            "point_in_time_recovery": [{"enabled": [True]}],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations
from typing import Any
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class RDSEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure all data stored in the RDS is securely encrypted at rest"
        id = "CKV_AWS_000"
        supported_resources = ['aws_db_instance']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "storage_encrypted"

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        result = super().scan_resource_conf(conf=conf)
        provider_name = conf.get("provider")
        if provider_name and isinstance(provider_name, list):
            providers = [g[1] for g in self.graph.nodes() if g[1].get('block_type_') == 'provider']
            provider = next((prov for prov in providers if prov[CustomAttributes.BLOCK_NAME] == provider_name[0]), None)
            if provider and provider.get("use_fips_endpoint") is True:
                return CheckResult.PASSED
        return result


check = RDSEncryption()



import os
import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.runner import Runner
from checkov.terraform.checks.resource.azure.AzureBatchAccountEndpointAccessDefaultAction import check
from tests.common.check_assertion_utils import checks_report_assertions


class TestAzureBatchAccountEndpointAccessDefaultAction(unittest.TestCase):

    def test(self):
        runner = Runner()

        test_files_dir = Path(__file__).parent / "example_AzureBatchAccountEndpointAccessDefaultAction"
        report = runner.run(root_folder=str(test_files_dir),
                            runner_filter=RunnerFilter(checks=[check.id]))

        passing_resources = {
            'azurerm_batch_account.pass_no_publicNetworkAccess',
            'azurerm_batch_account.pass_publicNetworkAccess_disabled',
            'azurerm_batch_account.pass_publicNetworkAccess_enabled_no_network_profile',
            'azurerm_batch_account.pass_publicNetworkAccess_enabled_no_account_access',
            'azurerm_batch_account.pass_publicNetworkAccess_enabled_default_action_deny',

        }
        failing_resources = {
            'azurerm_batch_account.fail_publicNetworkAccess_enabled_default_action_allow',
            'azurerm_batch_account.fail_bad_default_action_no_public_network',
        }

        # then
        checks_report_assertions(self, report, passing_resources, failing_resources)


if __name__ == '__main__':
    unittest.main()


import os
import unittest
from unittest.mock import patch

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.integration_features.features.vulnerabilities_integration import \
    VulnerabilitiesIntegration, NORMALIZE_PREFIX
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
from checkov.common.output.record import Record, SCA_PACKAGE_SCAN_CHECK_NAME
from checkov.common.output.report import Report
from checkov.common.sast.consts import SastLanguages
from checkov.sast.report import SastReport
from checkov.common.sast.report_types import Package, File, Function, PrismaReport


class TestVulnerabilitiesIntegration(unittest.TestCase):

    @patch.dict('os.environ', {'CKV_ENABLE_UPLOAD_SAST_IMPORTS':'True', 'CKV_ENABLE_SCA_INTEGRATE_SAST': 'True'})
    def test_full_enrich_cves(self):
        instance = BcPlatformIntegration()

        vul_integration = VulnerabilitiesIntegration(instance)

        cve1 = Record(check_id='CKV_CVE_2022_38778', check_name=SCA_PACKAGE_SCAN_CHECK_NAME, file_path='/package.json',
                      vulnerability_details={
                          'id': 'CVE-2022-38778', 'severity': 'medium', 'package_name': 'decode-uri-component',
                          'package_version': '0.2.0',
                          'risk_factors': {'Severity': 'Medium', 'HasFix': True, 'DoS': False,
                                           'AttackVector': 'network', 'AttackComplexity': 'low', 'IsUsed': False}
                      }, check_result=None, code_block=None, file_line_range=None, resource=None, evaluations=None,
                      check_class=None, file_abs_path='')
        cve2 = Record(check_id='CKV_CVE_2022_38778', check_name=SCA_PACKAGE_SCAN_CHECK_NAME, file_path='/package.json',
                      vulnerability_details={
                          'id': 'CVE-2022-11111', 'severity': 'medium', 'package_name': 'decode-uri-component',
                          'package_version': '0.2.0',
                          'risk_factors': {'Severity': 'Medium', 'HasFix': True, 'DoS': False,
                                           'AttackVector': 'network', 'AttackComplexity': 'low', 'IsUsed': False}
                      }, check_result=None, code_block=None, file_line_range=None, resource=None, evaluations=None,
                      check_class=None, file_abs_path='')
        cve3 = Record(check_id='CKV_CVE_2022_38778', check_name=SCA_PACKAGE_SCAN_CHECK_NAME,
                      file_path='/no_exists/package.json',
                      vulnerability_details={
                          'id': 'CVE-2022-22222', 'severity': 'medium', 'package_name': 'decode-uri-component',
                          'package_version': '0.2.0',
                          'risk_factors': {'Severity': 'Medium', 'HasFix': True, 'DoS': False,
                                           'AttackVector': 'network',
                                           'AttackComplexity': 'low', 'IsUsed': False}
                      }, check_result=None, code_block=None, file_line_range=None, resource=None, evaluations=None,
                      check_class=None, file_abs_path='')
        failed_checks: list[Record] = [cve1, cve2, cve3]
        sast_imports = {
                        '/innerFiles/code.js': {'all': ["bson", "decode-uri-component", "parse-path"]},
                        '/main.js': {'all': ["bson", "decode-uri-component", "parse-path"]}
                        }

        sca_report: Report = Report(check_type=CheckType.SCA_PACKAGE)
        sca_report.failed_checks = failed_checks
        sast_report: SastReport = SastReport(check_type=CheckType.SAST_JAVASCRIPT,
                                             language=SastLanguages.JAVASCRIPT, metadata=None,
                                             sast_report=PrismaReport(rule_match={}, errors={}, profiler={},
                                                                      run_metadata={}, imports={},
                                                                      reachability_report={},
                                                                      skipped_checks_by_file={}))
        sast_report.sast_imports = sast_imports
        merged_reports = [sca_report, sast_report]

        vul_integration.merge_sca_and_sast_reports(merged_reports)

        self.assertTrue(cve1.vulnerability_details.get('risk_factors', {})['IsUsed'])
        self.assertTrue(cve2.vulnerability_details.get('risk_factors', {})['IsUsed'])
        self.assertFalse(cve3.vulnerability_details.get('risk_factors', {})['IsUsed'])

    @patch.dict('os.environ', {'CKV_ENABLE_UPLOAD_SAST_IMPORTS':'True', 'CKV_ENABLE_SCA_INTEGRATE_SAST': 'True'})
    def test_unsupported_sast_lang(self):
        instance = BcPlatformIntegration()

        vul_integration = VulnerabilitiesIntegration(instance)

        cve1 = Record(check_id='CKV_CVE_2022_38778', check_name=SCA_PACKAGE_SCAN_CHECK_NAME, file_path='/csproj',
                      vulnerability_details={
                          'id': 'CVE-2022-38778', 'severity': 'medium', 'package_name': 'decode-uri-component',
                          'package_version': '0.2.0',
                          'risk_factors': {'Severity': 'Medium', 'HasFix': True, 'DoS': False,
                                           'AttackVector': 'network', 'AttackComplexity': 'low', 'IsUsed': False}
                      }, check_result=None, code_block=None, file_line_range=None, resource=None, evaluations=None,
                      check_class=None, file_abs_path='')

        failed_checks: list[Record] = [cve1]
        sast_imports = {
            'Imports': {'/innerFiles/code.js': {'All': ["'bson'", "'decode-uri-component'", "'parse-path'"]},
                        '/main.js': {'All': ["'bson'", "'decode-uri-component'", "'parse-path'"]}}}

        sca_report: Report = Report(check_type=CheckType.SCA_PACKAGE)
        sca_report.failed_checks = failed_checks
        sast_report: SastReport = SastReport(check_type=CheckType.SAST_JAVASCRIPT,
                                             language=SastLanguages.JAVASCRIPT, metadata=None,
                                             sast_report=PrismaReport(rule_match={}, errors={}, profiler={},
                                                                      run_metadata={}, imports={},
                                                                      reachability_report={},
                                                                      skipped_checks_by_file={}))
        sast_report.sast_imports = sast_imports
        merged_reports = [sca_report, sast_report]

        vul_integration.merge_sca_and_sast_reports(merged_reports)

        self.assertFalse(cve1.vulnerability_details.get('risk_factors', {})['IsUsed'])


    def test_compare_paths_same_level(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        main_file = '/package.json';
        relative_file = '/main.js';
        is_relative = vul_integration.is_deeper_or_equal_level(main_file, relative_file)
        self.assertTrue(is_relative)

    def test_compare_paths_child_level(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        main_file = '/package.json';
        relative_file = '/src/main.js';
        is_relative = vul_integration.is_deeper_or_equal_level(main_file, relative_file)
        self.assertTrue(is_relative)

    def test_compare_paths_parent_level(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        main_file = '/src/package.json';
        relative_file = '/main.js';
        is_relative = vul_integration.is_deeper_or_equal_level(main_file, relative_file)
        self.assertFalse(is_relative)

    def test_compare_paths_relative_level(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        main_file = '/package.json';
        relative_file = '../main.js';
        is_relative = vul_integration.is_deeper_or_equal_level(main_file, relative_file)
        self.assertFalse(is_relative)

    def test_compare_paths_valid_relative_level(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        main_file = '/src2/../src/main.js';
        relative_file = '/src/package.json';
        is_relative = vul_integration.is_deeper_or_equal_level(main_file, relative_file)
        self.assertTrue(is_relative)

    def test_normalized_package_name_case_dot(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        original = '../asdas/asdasd/asdasd/asd.txt'
        expected = f"{NORMALIZE_PREFIX}asd.txt"
        result = vul_integration.normalize_package_name(original)
        self.assertTrue(result, expected)

    def test_normalized_package_name_case_without_relative(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        original = 'asdas/asdasd/asdasd/asd.txt'
        expected = f"{NORMALIZE_PREFIX}asdas/asdasd/asdasd/asd.txt"
        result = vul_integration.normalize_package_name(original)
        self.assertTrue(result, expected)

    def test_normalized_package_name_case_dot_in_name(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        original = 'asd.txt'
        expected = f"{NORMALIZE_PREFIX}asd.txt"
        result = vul_integration.normalize_package_name(original)
        self.assertTrue(result, expected)

    def test_normalized_package_name_case_relative_package(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        original = '../asdas/asdasd/asdasd/asd.txt'
        expected = f"{NORMALIZE_PREFIX}asd.txt"
        result = vul_integration.normalize_package_name(original)
        self.assertTrue(result, expected)

    def test_normalized_package_name_case_with_underscore(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        original = 'asd2_asd'
        expected = f"{NORMALIZE_PREFIX}asd2asd"
        result = vul_integration.normalize_package_name(original)
        self.assertTrue(result, expected)

    def test_normalized_package_name_case_with_minus(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        original = 'asd2-asd'
        expected = f"{NORMALIZE_PREFIX}asd2asd"
        result = vul_integration.normalize_package_name(original)
        self.assertTrue(result, expected)

    def test_normalized_package_name_case_simple(self):
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        original = 'asd'
        expected = f"{NORMALIZE_PREFIX}asd"
        result = vul_integration.normalize_package_name(original)
        self.assertTrue(result, expected)

    def test_create_reachable_cves_by_package_map(self):
        filtered_reachability_entries = [
            ('/index.js', File(packages={
                'axios': Package(alias='ax', functions=[
                    Function(name='trim', alias='hopa', line_number=4, code_block='hopa()', cve_id='cve-11')
                ]),
                'lodash': Package(alias='', functions=[
                    Function(name='template', alias='', line_number=1, code_block='template()', cve_id='cve-12'),
                    Function(name='toNumber', alias='', line_number=4, code_block='hopa()', cve_id='cve-13')
                ])
            }))
        ]
        instance = BcPlatformIntegration()
        vul_integration = VulnerabilitiesIntegration(instance)
        reachable_data_by_package_map = vul_integration.create_reachable_cves_by_package_map(filtered_reachability_entries)
        assert reachable_data_by_package_map == {
            'axios': {'cve-11'},
            'lodash': {'cve-12', 'cve-13'}
        }


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

import os
from pathlib import Path
from typing import Any, TYPE_CHECKING

from checkov.serverless.utils import ServerlessElements

if TYPE_CHECKING:
    from checkov.serverless.graph_builder.graph_components.blocks import ServerlessBlock


def convert_graph_vertices_to_definitions(vertices: list[ServerlessBlock], root_folder: str | Path | None) \
        -> tuple[dict[str, dict[str, Any]], dict[str, dict[str, Any]]]:
    serverless_definitions: dict[str, dict[str, Any]] = {}
    breadcrumbs: dict[str, dict[str, Any]] = {}
    for vertex in vertices:
        block_path = vertex.path
        element_name = vertex.name.split('.')[-1]
        # Plugins section is formatted as a list
        if vertex.block_type == ServerlessElements.PLUGINS:
            serverless_definitions.setdefault(block_path, {}).setdefault(vertex.block_type, []).append(element_name)

        # If there is a key named 'value' in the config it means that
        # this vertex's config contains only a single string
        elif 'value' in vertex.config:
            # If the vertex is provider or service and it only contains a string the section should look like:
            # provider: <value>
            # service: <value>
            if element_name == ServerlessElements.PROVIDER or element_name == ServerlessElements.SERVICE:
                serverless_definitions.setdefault(block_path, {})[vertex.block_type] = vertex.config['value']

            # Otherwise it's a vertex of a specific nested attribute and need to include the full path
            # Examples:
            # provider:
            #   runtime: nodejs20.x
            # custom:
            #   myCustomVar: value
            else:
                serverless_definitions.setdefault(block_path, {}).setdefault(vertex.block_type, {})[element_name] = \
                    vertex.config['value']

        # Otherwise, the vertex config is a dict
        else:
            serverless_definitions.setdefault(block_path, {}).setdefault(vertex.block_type, {})[
                element_name] = vertex.config

        if vertex.breadcrumbs:
            relative_block_path = f"/{os.path.relpath(block_path, root_folder)}"
            add_breadcrumbs(vertex, breadcrumbs, relative_block_path)
    return serverless_definitions, breadcrumbs


def add_breadcrumbs(vertex: ServerlessBlock, breadcrumbs: dict[str, dict[str, Any]], relative_block_path: str) -> None:
    breadcrumbs.setdefault(relative_block_path, {})[vertex.name] = vertex.breadcrumbs



import pytest

from checkov.terraform.module_loading.loaders.versions_parser import get_version_constraints


@pytest.mark.parametrize(
    "input_str,expected",
    [
        ("1.2.0", "=1.2.0"),
        ("1.2.0-rc1", "=1.2.0rc1"),
        (">= 1.2.0, < 2.0.0", ">=1.2.0,<2.0.0"),
        ("not-a-version", ""),
        ("<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=>=1.2.3", ""),
    ],
    ids=["sem_ver", "pre_release", "multi", "not_a_version", "back_tracking"],
)
def test_get_version_constraints(input_str: str, expected: str) -> None:
    result = get_version_constraints(input_str)

    assert ",".join(str(version) for version in result) == expected



import unittest

from checkov.terraform.checks.resource.aws.PasswordPolicyReuse import check
from checkov.common.models.enums import CheckResult


class TestPasswordPolicyReuse(unittest.TestCase):

    def test_success(self):
        resource_conf = {
            "minimum_password_length": [15],
            "require_lowercase_characters": [True],
            "require_numbers": [True],
            "require_uppercase_characters": [True],
            "require_symbols": [True],
            "allow_users_to_change_password": [True],
            "max_password_age": [89],
            "password_reuse_prevention": [24]
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure(self):
        resource_conf = {
            "minimum_password_length": [15],
            "require_lowercase_characters": [True],
            "require_numbers": [True],
            "require_uppercase_characters": [True],
            "require_symbols": [True],
            "allow_users_to_change_password": [True],
            "max_password_age": [89],
            "password_reuse_prevention": [4]
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_on_missing_property(self):
        resource_conf = {
            "require_numbers": [True],
            "require_symbols": [True],
            "allow_users_to_change_password": [True],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_another_test(self):
        conf = {'count': ['True ? 1 : 0}'], 'max_password_age': [0], 'minimum_password_length': [8], 'allow_users_to_change_password': [True], 'hard_expiry': [False], 'password_reuse_prevention': ['${var.password_reuse_prevention}'], 'require_lowercase_characters': [True], 'require_uppercase_characters': [True], 'require_numbers': [True], 'require_symbols': [True]}

        scan_result = check.scan_resource_conf(conf=conf)
        self.assertEqual(CheckResult.UNKNOWN, scan_result)


if __name__ == '__main__':
    unittest.main()



import pytest

from checkov.common.graph.graph_builder.utils import update_dictionary_attribute, join_double_quote_surrounded_dot_split

@pytest.mark.parametrize(
    "input_parts,expected_parts",
    [
        (
            ["google_project_iam_binding", 'role["roles/logging', 'admin"]'],
            ["google_project_iam_binding", 'role["roles/logging.admin"]'],
        ),
        (
            ["module", "google_project_iam_binding", 'role["roles/logging', 'admin"]'],
            ["module", "google_project_iam_binding", 'role["roles/logging.admin"]'],
        ),
        (
            [
                "module",
                "google_project_iam_binding",
                'role["roles/logging',
                'admin"]',
                "module",
                "google_project_iam_binding",
                'role["roles/logging',
                'admin"]',
            ],
            [
                "module",
                "google_project_iam_binding",
                'role["roles/logging.admin"]',
                "module",
                "google_project_iam_binding",
                'role["roles/logging.admin"]',
            ],
        ),
    ],
    ids=["resource", "module_resource", "complex"],
)
def test_join_double_quote_surrounded_dot_split(input_parts, expected_parts):
    assert join_double_quote_surrounded_dot_split(str_parts=input_parts) == expected_parts

def test_update_dictionary_attribute_nested():
    origin_config = {'aws_s3_bucket': {
        'destination': {'bucket': ['tf-test-bucket-destination-12345'], 'acl': ['${var.acl}'],
                        'versioning': [{'enabled': ['${var.is_enabled}']}]}}}
    key_to_update = 'versioning.enabled'
    new_value = [False]
    expected_config = {'aws_s3_bucket': {
        'destination': {'bucket': ['tf-test-bucket-destination-12345'], 'acl': ['${var.acl}'],
                        'versioning': [{'enabled': [False]}]}}}
    actual_config = update_dictionary_attribute(origin_config, key_to_update, new_value)
    assert expected_config == actual_config, f'failed to update config. expected: {expected_config}, got: {actual_config}'


def test_update_dictionary_attribute():
    origin_config = {'aws_s3_bucket': {
        'destination': {'bucket': ['tf-test-bucket-destination-12345'], 'acl': ['${var.acl}'],
                        'versioning': [{'enabled': ['${var.is_enabled}']}]}}}
    key_to_update = 'acl'
    new_value = ['public-read']
    expected_config = {'aws_s3_bucket': {
        'destination': {'bucket': ['tf-test-bucket-destination-12345'], 'acl': ['public-read'],
                        'versioning': [{'enabled': ['${var.is_enabled}']}]}}}
    actual_config = update_dictionary_attribute(origin_config, key_to_update, new_value)
    assert expected_config == actual_config, f'failed to update config.\nexpected: {expected_config}\ngot: {actual_config}'


def test_update_dictionary_locals():
    origin_config = {'aws_s3_bucket': {
        'destination': {'bucket': ['tf-test-bucket-destination-12345'], 'acl': ['${var.acl}'],
                        'versioning': [{'enabled': ['${var.is_enabled}']}]}}}
    key_to_update = 'acl'
    new_value = ['public-read']
    expected_config = {'aws_s3_bucket': {
        'destination': {'bucket': ['tf-test-bucket-destination-12345'], 'acl': ['public-read'],
                        'versioning': [{'enabled': ['${var.is_enabled}']}]}}}
    actual_config = update_dictionary_attribute(origin_config, key_to_update, new_value)
    assert expected_config == actual_config, f'failed to update config.\nexpected: {expected_config}\ngot: {actual_config}'



import json
import os
from unittest import TestCase, mock
from parameterized import parameterized_class

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.terraform.runner import Runner

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))

@parameterized_class([
   {"db_connector": NetworkxConnector},
   {"db_connector": RustworkxConnector}
])
class TestGraphBuilder(TestCase):
    @mock.patch.dict(os.environ, {"CHECKOV_NEW_TF_PARSER": "False"})
    def test_build_graph(self):
        resources_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "resources", "graph_files_test")
        source_files = ["pass_s3.tf", "variables.tf"]
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(None, None, files=list(map(lambda f: f'{resources_path}/{f}', source_files)))
        tf_definitions = runner.definitions
        self.assertEqual(5, len(report.failed_checks))
        for file, definitions in tf_definitions.items():
            if file.file_path.endswith('pass_s3.tf'):
                s3_bucket_config = definitions['resource'][0]['aws_s3_bucket']['bucket_with_versioning']
                # Evaluation succeeded for included vars
                self.assertTrue(s3_bucket_config['versioning'][0]['enabled'][0])
                # Evaluation does not run for un-included vars
                self.assertEqual(s3_bucket_config['server_side_encryption_configuration'][0]['rule'][0]['apply_server_side_encryption_by_default'][0]['sse_algorithm'][0], 'var.encryption')

    def test_build_graph_new_tf_module(self):
        resources_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "resources", "graph_files_test")
        source_files = ["pass_s3.tf", "variables.tf"]
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(None, None, files=list(map(lambda f: f'{resources_path}/{f}', source_files)))
        tf_definitions = runner.definitions
        self.assertEqual(5, len(report.failed_checks))
        for file, definitions in tf_definitions.items():
            if file.file_path.endswith('pass_s3.tf'):
                s3_bucket_config = definitions['resource'][0]['aws_s3_bucket']['bucket_with_versioning']
                # Evaluation succeeded for included vars
                self.assertTrue(s3_bucket_config['versioning'][0]['enabled'][0])
                # Evaluation does not run for un-included vars
                self.assertEqual(s3_bucket_config['server_side_encryption_configuration'][0]['rule'][0]['apply_server_side_encryption_by_default'][0]['sse_algorithm'][0], 'var.encryption')

    def test_run_clean(self):
        resources_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "resources", "graph_files_test")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=resources_path)
        self.assertEqual(6, len(report.failed_checks))
        self.assertEqual(5, len(report.passed_checks))
        self.assertEqual(0, len(report.skipped_checks))

    def test_module_and_variables(self):
        resources_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "resources", "modules-and-vars")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=resources_path)
        self.assertLessEqual(2, len(report.failed_checks))
        self.assertLessEqual(12, len(report.passed_checks))
        self.assertEqual(0, len(report.skipped_checks))

        found_versioning_failure = False

        for record in report.failed_checks:
            if record.check_id != 'CKV_AWS_40':
                self.assertIsNotNone(record.breadcrumbs)
            if record.check_id == 'CKV_AWS_21':
                found_versioning_failure = True
                bc = record.breadcrumbs.get('versioning.enabled')
                self.assertEqual(len(bc), 2)
                bc = bc[0]
                self.assertEqual(bc.get('type'), 'module')
                self.assertEqual(os.path.relpath(bc.get('path'), resources_path), 'examples/complete/main.tf')
                self.assertEqual(record.resource, 'module.s3_bucket.aws_s3_bucket.default')

        self.assertTrue(found_versioning_failure)



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.FunctionAppsEnableAuthentication import check
from checkov.common.models.enums import CheckResult


class TestFunctionAppsEnableAuthentication(unittest.TestCase):

    def test_failure_missing_authentication_block(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_function_app" "example" {
              name                       = "test-azure-functions"
              location                   = "azurerm_resource_group.example.location"
              resource_group_name        = "azurerm_resource_group.example.name"
              app_service_plan_id        = "azurerm_app_service_plan.example.id"
              storage_account_name       = "azurerm_storage_account.example.name"
              storage_account_access_key = "azurerm_storage_account.example.primary_access_key"
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_function_app']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_function_app" "example" {
              name                       = "test-azure-functions"
              location                   = "azurerm_resource_group.example.location"
              resource_group_name        = "azurerm_resource_group.example.name"
              app_service_plan_id        = "azurerm_app_service_plan.example.id"
              storage_account_name       = "azurerm_storage_account.example.name"
              storage_account_access_key = "azurerm_storage_account.example.primary_access_key"
              auth_settings {
                enabled = true
              }
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_function_app']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failed(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_function_app" "example" {
              name                       = "test-azure-functions"
              location                   = "azurerm_resource_group.example.location"
              resource_group_name        = "azurerm_resource_group.example.name"
              app_service_plan_id        = "azurerm_app_service_plan.example.id"
              storage_account_name       = "azurerm_storage_account.example.name"
              storage_account_access_key = "azurerm_storage_account.example.primary_access_key"
              auth_settings {
                enabled = false
              }
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_function_app']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == '__main__':
    unittest.main()



import os

from parameterized import parameterized_class

from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestIsFalse(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestIsFalse, self).setUp()

    def test_is_false(self):
        root_folder = '../../../resources/boolean_test'
        check_id = "FalseValue"
        should_pass = ['azurerm_storage_account.fail3', 'azurerm_storage_account.pass1', 'azurerm_storage_account.pass2']
        should_fail = ['azurerm_storage_account.fail1', 'azurerm_storage_account.fail2']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        super(TestIsFalse, self).run_test(root_folder=root_folder, expected_results=expected_results,
                                               check_id=check_id)

    def test_is_true(self):
        root_folder = '../../../resources/encryption'
        check_id = "TrueValue"
        should_pass = []
        should_fail = ['aws_neptune_cluster.unencrypted_neptune', 'aws_neptune_cluster.encrypted_neptune']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        super(TestIsFalse, self).run_test(root_folder=root_folder, expected_results=expected_results,
                                               check_id=check_id)


import unittest

import hcl2

from checkov.terraform.checks.resource.gcp.GoogleComputeFirewallUnrestrictedIngress3389 import check, PORT
from checkov.common.models.enums import CheckResult


class TestGoogleComputeFirewallUnrestrictedIngress3389(unittest.TestCase):

    def test_failure(self):
        resource_conf = {'name': ['${var.name}-${var.region}-mesos-ssh'],
                         'network': ['${google_compute_network.mesos-global-net.name}'],
                         'allow': [{'protocol': ['tcp'], 'ports': [[str(PORT)]]}], 'target_tags': [['ssh']],
                         'source_ranges': [['0.0.0.0/0']]}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_1(self):
        resource_conf = {'name': ['${var.name}-${var.region}-mesos-ssh'],
                         'network': ['${google_compute_network.mesos-global-net.name}'],
                         'allow': [{'protocol': ['tcp'], 'ports': [[
                             str(PORT)]]}], 'target_tags': [['ssh']], 'source_ranges': [['172.1.2.3/32']]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_2(self):
        hcl_res = hcl2.loads("""
                    resource "google_compute_firewall" "no-allow-blocks" {
                      name        = "deny-all-egress-all"
                      description = "Prevent all egress traffic by default"
                      disabled = true
                    
                      network        = google_compute_network.vpc_network.name
                      enable_logging = true
                    
                      priority           = 65534
                      direction          = "EGRESS"
                      destination_ranges = ["0.0.0.0/0"]
                      deny { protocol = "all" }
                    }
                        """)
        resource_conf = hcl_res['resource'][0]['google_compute_firewall']['no-allow-blocks']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()
